--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity CLKGEN is
	port (
		CLK50MIN	: in  std_logic;	-- clock 50MHz input
		RSTN		: in  std_logic;
		CLK66MOUT	: out std_logic;
		CLK100MOUT	: out std_logic;
		CLK100M_DI	: out std_logic;
		CLK1SOUT	: out std_logic;
		LOCK_PLL	: out std_logic
	);
end CLKGEN;

architecture RTL of CLKGEN is

	component PLL66M100M is
		port (
			ARESET	: in  std_logic;
			INCLK0	: in  std_logic;
			C0		: out std_logic;
			C1		: out std_logic;
			C2		: out std_logic;
			LOCKED	: out std_logic
		);
	end component;

	signal aclr			: std_logic;
	signal rstcnt1s		: std_logic_vector(25 downto 0);

	signal clk66m		: std_logic;
	signal clk100m		: std_logic;
	signal clk100m_dly	: std_logic;

	signal lock_66m		: std_logic;

	signal clk1scntlow	: std_logic_vector(15 downto 0);
	signal clk1scnthigh	: std_logic_vector(9 downto 0);
	signal clk1sout_i	: std_logic;

begin

	U_PLL66M100M : PLL66M100M
	port map (
		ARESET	=> aclr,
		INCLK0	=> CLK50MIN,
		C0		=> clk66m,
		C1		=> clk100m,
		C2		=> clk100m_dly,
		LOCKED	=> lock_66m
	);

	process (CLK50MIN,RSTN)
	begin
		if (RSTN = '0') then
			aclr     <= '1';
			rstcnt1s <= (others => '0');
		elsif (CLK50MIN'event and CLK50MIN = '1') then
			if (rstcnt1s = 49999999) then
				aclr <= '0';
			else
				rstcnt1s <= rstcnt1s + 1;
			end if;
		end if;
	end process;

	process (clk100m,RSTN)
	begin
		if (RSTN = '0') then
			clk1scntlow <= (others => '0');
		elsif (clk100m'event and clk100m = '1') then
			if (clk1scntlow = 49999) then
				clk1scntlow <= (others => '0');
			else
				clk1scntlow <= clk1scntlow + 1;
			end if;
		end if;
	end process;

	process (clk100m,RSTN)
	begin
		if (RSTN = '0') then
			clk1scnthigh <= (others => '0');
			clk1sout_i <= '0';
		elsif (clk100m'event and clk100m = '1') then
			if (clk1scntlow = 49999) then
				if (clk1scnthigh = 999) then
					clk1scnthigh <= (others => '0');
					clk1sout_i <= not clk1sout_i;
				else
					clk1scnthigh <= clk1scnthigh + 1;
				end if;
			end if;
		end if;
	end process;

	CLK100MOUT  <= clk100m;
	CLK100M_DI  <= clk100m_dly;
	CLK66MOUT   <= clk66m;
	CLK1SOUT    <= clk1sout_i;

	LOCK_PLL    <= lock_66m;

end RTL;
