--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity INTGEN is
	port (
		RDN		: in  std_logic;
		CLK		: in  std_logic;
		RSTN	: in  std_logic;
		INTRQN	: out std_logic;
		DO		: out std_logic_vector(7 downto 0)
	);
end INTGEN;

architecture RTL of INTGEN is

	signal rdn_f1		: std_logic;
	signal rdn_f2		: std_logic;

	signal tim25mcnt	: std_logic_vector(21 downto 0);
	signal intrqn_i		: std_logic;

	signal datao_i		: std_logic_vector(7 downto 0);

begin

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			rdn_f1 <= '1';
			rdn_f2 <= '1';
		elsif (CLK'event and CLK = '1') then
			rdn_f1 <= RDN;
			rdn_f2 <= rdn_f1;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			tim25mcnt <= (others => '1');
		elsif (CLK'event and CLK = '1') then
			if (rdn_f1 = '1' and rdn_f2 = '0') then
				tim25mcnt <= (others => '0');
			elsif (tim25mcnt /= 1666666) then
				tim25mcnt <= tim25mcnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			intrqn_i <= '1';
		elsif (CLK'event and CLK = '1') then
			if (rdn_f1 = '1' and rdn_f2 = '0') then
				intrqn_i <= '1';
			elsif (tim25mcnt = 1666666) then
				intrqn_i <= '0';
			end if;
		end if;
	end process;


	datao_i <= X"04";		-- interrupt vector

	INTRQN <= intrqn_i;
	DO     <= datao_i;

end RTL;
