--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity MEMSEL is
	port (
		CPUADD		: in  std_logic_vector(15 downto 0);
		CPUDT		: in  std_logic_vector(7 downto 0);
		CPUMREQN	: in  std_logic;
		CPUIORQN	: in  std_logic;
		CPUWRN		: in  std_logic;
		CPURDN		: in  std_logic;
		CPUEXCASN	: in  std_logic;
		CPURAS2N	: in  std_logic;
		CPUDRD2N	: in  std_logic;
		CPUWEN		: in  std_logic;
		CSN			: in  std_logic;
		SRAM_ENB	: in  std_logic;
		CTRLRAMADD	: in  std_logic_vector(19 downto 0);
		CTRLRAMWRN	: in  std_logic;
		CTRLBANKADD	: in  std_logic_vector(2 downto 0);
		CTRLBANKDT	: in  std_logic_vector(7 downto 0);
		CTRLBANKWRN	: in  std_logic;
		USEROM		: in  std_logic_vector(1 downto 0);
		RSTP6N		: in  std_logic;
		CLK			: in  std_logic;
		RSTN		: in  std_logic;
		SDRAMADD	: out std_logic_vector(19 downto 0);
		SDRAM_WRN	: out std_logic;
		SDRAM_RDN	: out std_logic;
		PC60MODE	: out std_logic;
		ROMXMODE	: out std_logic;
		CS01R		: out std_logic;
		CS01RX		: out std_logic;
		CSCG		: out std_logic;
		SRAM_OUTENB	: out std_logic;
		SRAM_DO		: out std_logic_vector(7 downto 0);
		DO			: out std_logic_vector(7 downto 0)
	);
end MEMSEL;

architecture RTL of MEMSEL is

	signal romset		: std_logic_vector(1 downto 0);
	signal romset_init	: std_logic_vector(1 downto 0);
	signal ramset		: std_logic_vector(1 downto 0);
	signal ramset_init	: std_logic_vector(1 downto 0);

	type bank_type is array (0 to 7) of std_logic_vector(5 downto 0);
	signal bankset		: bank_type;
	signal bankset_init	: bank_type;

	signal pc60mode_i	: std_logic;
	signal romxmode_i	: std_logic;
	signal cs01r_i		: std_logic;
	signal cs01rx_i		: std_logic;
	signal cscg_i		: std_logic;
	signal cs01w_i		: std_logic;
	signal cs02w_i		: std_logic;
	signal port06r		: std_logic;


	signal bsel			: std_logic_vector(5 downto 0);
	signal bvdg			: std_logic_vector(5 downto 0);

	signal sdramadd_i	: std_logic_vector(19 downto 0);
	signal sdram_wrn_i	: std_logic;
	signal sdram_rdn_i	: std_logic;

	signal bank_do		: std_logic_vector(5 downto 0);
	signal do_i			: std_logic_vector(7 downto 0);

	signal sram_wr		: std_logic;
	signal sram_do_i	: std_logic_vector(7 downto 0);

	component SRAM16386W8B
		port (
			address	: in  std_logic_vector(13 downto 0);
			data	: in  std_logic_vector(7 downto 0);
			wren	: in  std_logic;
			clock	: in  std_logic;
			q		: out std_logic_vector(7 downto 0)
		);
	end component;

begin

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			bankset_init(0) <= "010000";		-- RAM bank0
			bankset_init(1) <= "010001";		-- RAM bank1
			bankset_init(2) <= "000000";		-- ROM bank0
			bankset_init(3) <= "000001";		-- ROM bank1
			bankset_init(4) <= "010100";		-- RAM bank4
			bankset_init(5) <= "010101";		-- RAM bank5
			bankset_init(6) <= "010110";		-- RAM bank6
			bankset_init(7) <= "010111";		-- RAM bank7
			romset_init   <= "00";
			ramset_init   <= "00";
		elsif (CLK'event and CLK = '1') then
			if (USEROM(1) = '1' or port06r = '1') then
				for i in 0 to 7 loop
					bankset_init(i) <= bankset(i);
				end loop;
				romset_init   <= romset;
				ramset_init   <= ramset;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			romset <= "00";
			ramset <= "00";
			bankset(0) <= "010000";		-- RAM bank0
			bankset(1) <= "010001";		-- RAM bank1
			bankset(2) <= "000000";		-- ROM bank0
			bankset(3) <= "000001";		-- ROM bank1
			bankset(4) <= "010100";		-- RAM bank4
			bankset(5) <= "010101";		-- RAM bank5
			bankset(6) <= "010110";		-- RAM bank6
			bankset(7) <= "010111";		-- RAM bank7
		elsif (CLK'event and CLK = '1') then
			if (RSTP6N = '0') then
				if (USEROM(0) = '1') then
					for i in 0 to 7 loop
						bankset(i) <= bankset_init(i);
					end loop;
					romset <= romset_init;
					ramset <= ramset_init;
				else
					romset <= "00";
					ramset <= "00";
					bankset(0) <= "010000";		-- RAM bank0
					bankset(1) <= "010001";		-- RAM bank1
					bankset(2) <= "010010";		-- RAM bank2
					bankset(3) <= "010011";		-- RAM bank3
					bankset(4) <= "010100";		-- RAM bank4
					bankset(5) <= "010101";		-- RAM bank5
					bankset(6) <= "010110";		-- RAM bank6
					bankset(7) <= "010111";		-- RAM bank7
				end if;
			elsif (CTRLBANKWRN = '0') then
				if    (CTRLBANKDT(7 downto 6) = "00") then
					romset <= CTRLBANKDT(5 downto 4);
				elsif (CTRLBANKDT(7 downto 6) = "01") then
					ramset <= CTRLBANKDT(5 downto 4);
				end if;
				bankset( conv_integer(CTRLBANKADD) ) <=
					CTRLBANKDT(7 downto 6) & CTRLBANKDT(3 downto 0);
			elsif (CPUIORQN = '0' and CPUWRN = '0') then
				if (CSN = '0') then							-- 0x30-0x3f
					if    (CPUDT(7 downto 6) = "00") then
						romset <= CPUDT(5 downto 4);
					elsif (CPUDT(7 downto 6) = "01") then
						ramset <= CPUDT(5 downto 4);
					end if;
					bankset( conv_integer(CPUADD(2 downto 0) ) ) <=
						CPUDT(7 downto 6) & CPUDT(3 downto 0);
				elsif (CPUADD(7 downto 4) = "0111") then	-- 0x70-0x7f
					bankset(2) <= "00" & CPUDT(3 downto 0); 
				end if;
			end if;
		end if;
	end process;

	bank_do <= bankset( conv_integer( CPUADD(2 downto 0) ) );

	do_i(7 downto 6) <= bank_do(5 downto 4);
	do_i(3 downto 0) <= bank_do(3 downto 0);

	do_i(5 downto 4) <= 
		romset when (bank_do(5 downto 4) = "00") else
		ramset when (bank_do(5 downto 4) = "01") else
		"11";


	bsel <= bankset( conv_integer(CPUADD(15 downto 13)) );

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sdramadd_i  <= (others => '0');
			sdram_wrn_i <= '1';
			sdram_rdn_i <= '1';
		elsif (CLK'event and CLK = '1') then
			if (CTRLRAMWRN = '0') then						 -- ExtROM/ExtRAM write from control CPU
				sdramadd_i  <= CTRLRAMADD;
				sdram_wrn_i <= '0';
				sdram_rdn_i <= '1';
			elsif (CPUMREQN = '0' and CPURDN = '0') then	-- ExtRAM read / ExtROM read
				if    (bsel(5 downto 4) = "00") then
					sdramadd_i  <= "0" & romset & bsel(3 downto 0) & CPUADD(12 downto 0);
					sdram_wrn_i <= '1';
					sdram_rdn_i <= '0';
				elsif (bsel(5 downto 4) = "01") then
					sdramadd_i  <= "1" & ramset & bsel(3 downto 0) & CPUADD(12 downto 0);
					sdram_wrn_i <= '1';
					sdram_rdn_i <= '0';
				else
					sdramadd_i  <= (others => '1');
					sdram_wrn_i <= '1';
					sdram_rdn_i <= '1';
				end if;
			elsif (
				(CPUEXCASN = '0' and CPURAS2N = '0' and CPUWEN = '0') or
				(CPUMREQN = '0' and CPUWRN = '0' and CPUADD(15 downto 14) = "00" and
					cs01w_i = '1' and pc60mode_i = '1') or
				(CPUMREQN = '0' and CPUWRN = '0' and CPUADD(15 downto 14) = "01" and
					cs02w_i = '1' and pc60mode_i = '1')
			) then	-- ExtRAM write
				if    (bsel(5 downto 4) = "00") then
					sdramadd_i  <= (others => '1');
					sdram_wrn_i <= '1';
					sdram_rdn_i <= '1';
				elsif (bsel(5 downto 4) = "01") then
					sdramadd_i  <= "1" & ramset & bsel(3 downto 0) & CPUADD(12 downto 0);
					sdram_wrn_i <= '0';
					sdram_rdn_i <= '1';
				else
					sdramadd_i  <= (others => '1');
					sdram_wrn_i <= '1';
					sdram_rdn_i <= '1';
				end if;
			else
				sdramadd_i  <= (others => '1');
				sdram_wrn_i <= '1';
				sdram_rdn_i <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			pc60mode_i <= '0';
			romxmode_i <= '0';
			cs01r_i    <= '0';
			cs01rx_i   <= '0';
			cscg_i     <= '0';
			cs01w_i    <= '0';
			cs02w_i    <= '0';
		elsif (CLK'event and CLK = '1') then
			if (RSTP6N = '0') then
				pc60mode_i <= '0';
				romxmode_i <= '0';
				cs01r_i    <= '0';
				cs01rx_i   <= '0';
				cscg_i     <= '0';
				cs01w_i    <= '0';
				cs02w_i    <= '0';
			elsif (CPUIORQN = '0' and CPUWRN = '0') then
				if (CPUADD(7 downto 0) = X"06") then		-- 0x06
					if (CPUDT = X"66") then
						pc60mode_i <= '1';
					else
						pc60mode_i <= '0';
					end if;
					if (CPUDT = X"77") then
						romxmode_i <= '1';
					else
						romxmode_i <= '0';
					end if;
				elsif (CPUADD(7 downto 0) = X"F0") then		-- 0xF0
					if (
						CPUDT(3 downto 0) = X"0" or
						CPUDT(3 downto 0) = X"1" or
						CPUDT(3 downto 0) = X"2" or
						CPUDT(3 downto 0) = X"5" or
						CPUDT(3 downto 0) = X"6" or
						CPUDT(3 downto 0) = X"F"
					) then
						cs01r_i <= '0';
					else
						cs01r_i <= '1';
					end if;
					if (CPUDT(3 downto 0) = X"1") then
						cs01rx_i <= '1';
					else
						cs01rx_i <= '0';
					end if;
				elsif (CPUADD(7 downto 0) = X"F2") then		-- 0xF2
					if (CPUDT(1 downto 0) = "00") then
						cs01w_i <= '0';
					else
						cs01w_i <= '1';
					end if;
					if (CPUDT(3 downto 2) = "00") then
						cs02w_i <= '0';
					else
						cs02w_i <= '1';
					end if;
				elsif (CPUADD(7 downto 0) = X"92") then		-- 0x92
					if    (CPUDT(2) = '0') then
						cscg_i <= '1';
					elsif (CPUDT(2) = '1') then
						cscg_i <= '0';
					end if;
				elsif (CPUADD(7 downto 0) = X"93") then		-- 0x93
					if    (CPUDT = X"04") then
						cscg_i <= '1';
					elsif (CPUDT = X"05") then
						cscg_i <= '0';
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			port06r    <= '0';
		elsif (CLK'event and CLK = '1') then
			if (CPUIORQN = '0' and CPURDN = '0') then
				if (CPUADD(7 downto 0) = X"06") then		-- 0x06
					port06r    <= '1';
				else
					port06r    <= '0';
				end if;
			else
				port06r    <= '0';
			end if;
		end if;
	end process;


	U_SRAM16384W8B : SRAM16386W8B
	port map (
		address	=> CPUADD(13 downto 0),
		data	=> CPUDT,
		wren	=> sram_wr,
		clock	=> CLK,
		q		=> sram_do_i
	);

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sram_wr <= '0';
		elsif (CLK'event and CLK = '1') then
			if ( CPUEXCASN = '0' and CPURAS2N = '0' and CPUWEN = '0' and SRAM_ENB = '1' ) then
				if (    CPUADD(15 downto 13) = "100" and bankset(4)(5 downto 4) = "01" ) then
					sram_wr <= '1';
				elsif ( CPUADD(15 downto 13) = "101" and bankset(5)(5 downto 4) = "01" ) then
					sram_wr <= '1';
				else
					sram_wr <= '0';
				end if;
			else
				sram_wr <= '0';
			end if;
		end if;
	end process;


	SDRAMADD  <= sdramadd_i;
	SDRAM_WRN <= sdram_wrn_i;
	SDRAM_RDN <= sdram_rdn_i;

	PC60MODE  <= pc60mode_i;
	ROMXMODE  <= romxmode_i;
	CS01R     <= cs01r_i;
	CS01RX    <= cs01rx_i;
	CSCG      <= cscg_i;

	SRAM_OUTENB <=
		'1' when (
			CPUMREQN = '1' and CPURDN = '1' and CPUEXCASN = '0' and
			CPURAS2N = '0' and CPUDRD2N = '0' and SRAM_ENB = '1' and
			CPUADD(13) = '0' and bankset(4)(5 downto 4) = "01") else
		'1' when (
			CPUMREQN = '1' and CPURDN = '1' and CPUEXCASN = '0' and
			CPURAS2N = '0' and CPUDRD2N = '0' and SRAM_ENB = '1' and
			CPUADD(13) = '1' and bankset(5)(5 downto 4) = "01") else
		'0';

	SRAM_DO <= sram_do_i;

	DO <= do_i;

end RTL;
