-- ROM/RAM cartridge for DE0-nano
--

-- 
-- ANZX[hƊe[q̏Ԃ𐮗Ă
--                         (nEXCAS)(nRAS2)(_DRD2)
-- _MREQ _RD _WR _WE _CS2/3  _CAS   _RAS   _VRD

--   0    1   0   0    1      0      0     1    Ext-RAM Write(P6/mk2/SR)
--   0    0   1   1    1      0      0     0    Ext-RAM Read (P6/mk2/SR)
--   1    1   1   1    1      0      0     0    Ext-RAM Read from VDP (P6)

--   0    1   0   0    1      0      1     1    Int-RAM Write(P6/mk2)
--   0    1   0   0    1      1      0     1    Int-RAM Write(SR)
--   0    0   1   1    1      0      1     1    Int-RAM Read (P6/mk2)
--   0    0   1   1    1      1      0     1    Int-RAM Read (SR)
--   1    1   1   1    1      0      1     1    Int-RAM Read from VDP (P6)

--   0    0   1   1    0      1      1     1    Ext-ROM Read(P6/mk2)
--   0    0   1   1    0      1      0     1    Ext-ROM Read(SR)

--   0    0   1   1    1      1      1     1    Int-ROM Read(P6/mk2)
--   0    0   1   1    1      1      0     1    Int-ROM Read(SR)


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity NANOCART is
	port (
		CLK50M			: in    std_logic;						-- clock 50MHz input #1
		LED				: out   std_logic_vector(7 downto 0);	-- LED
		BUTTON			: in    std_logic_vector(1 downto 0);	-- push button
		SW				: in    std_logic_vector(4 downto 1);	-- DIPSW
		DRAM_D			: inout std_logic_vector(15 downto 0);	-- SDRAM data bus
		DRAM_A			: out   std_logic_vector(12 downto 0);	-- SDRAM address bus
		DRAM_CLK		: out   std_logic;						-- SDRAM clock output
		DRAM_CKE		: out   std_logic;						-- SDRAM clock enable
		DRAM_LDQM		: out   std_logic;						-- SDRAM LowerByte Data Mask
		DRAM_UDQM		: out   std_logic;						-- SDRAM UpperByte Data Mask
		DRAM_WE_N		: out   std_logic;						-- SDRAM write Enable
		DRAM_CAS_N		: out   std_logic;						-- SDRAM CAS
		DRAM_RAS_N		: out   std_logic;						-- SDRAM RAS
		DRAM_CS_N		: out   std_logic;						-- SDRAM chip select
		DRAM_BA			: out   std_logic_vector(1 downto 0);	-- SDRAM Bank
		I2C_SCLK		: out   std_logic;						-- I2C clock output
		I2C_SDAT		: inout std_logic;						-- I2C data
		G_SENSOR_CS_N	: out   std_logic;						-- G-SENSOR CS output
		G_SENSOR_INT	: in    std_logic;						-- G-SENSOR data input
		ADC_CS_N		: out   std_logic;						-- A/D CS output
		ADC_SADDR		: out   std_logic;						-- A/D address output
		ADC_SCLK		: out   std_logic;						-- A/D clock output
		ADC_SDAT		: in    std_logic;						-- A/D data input
		GPIO0_D_00		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_01		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_02		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_03		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_04		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_05		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_06		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_07		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_08		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_09		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_10		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_11		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_12		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_13		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_14		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_15		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_16		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_17		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_18		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_19		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_20		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_21		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_22		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_23		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_24		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_25		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_26		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_27		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_28		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_29		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_30		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_31		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_32		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_33		: inout std_logic;						-- GPIO #0 data
		GPIO0_CLKIN_0	: in    std_logic;						-- GPIO #0 clock input
		GPIO0_CLKIN_1	: in    std_logic;						-- GPIO #0 clock input
		GPIO1_D_00		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_01		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_02		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_03		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_04		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_05		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_06		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_07		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_08		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_09		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_10		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_11		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_12		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_13		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_14		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_15		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_16		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_17		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_18		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_19		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_20		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_21		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_22		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_23		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_24		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_25		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_26		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_27		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_28		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_29		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_30		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_31		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_32		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_33		: inout std_logic;						-- GPIO #1 data
		GPIO1_CLKIN_0	: in    std_logic;						-- GPIO #1 clock input
		GPIO1_CLKIN_1	: in    std_logic;						-- GPIO #1 clock input
		GPIO2_D_00		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_01		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_02		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_03		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_04		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_05		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_06		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_07		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_08		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_09		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_10		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_11		: inout std_logic;						-- GPIO #2 data
		GPIO2_D_12		: inout std_logic;						-- GPIO #2 data
		GPIO2_CLKIN_0	: in    std_logic;						-- GPIO #2 clock input
		GPIO2_CLKIN_1	: in    std_logic;						-- GPIO #2 clock input
		GPIO2_CLKIN_2	: in    std_logic						-- GPIO #2 clock input
	);
end NANOCART;

architecture RTL of NANOCART is

-- PC-6001 I/F
	signal P6_A			: std_logic_vector(15 downto 0);
	signal P6_D_i		: std_logic_vector(7 downto 0);
	signal P6_D_o		: std_logic_vector(7 downto 0);
	signal P6_DOUTcnt	: std_logic;
	signal P6_nM1		: std_logic;
	signal P6_nIORQ		: std_logic;
	signal P6_nMREQ		: std_logic;
	signal P6_nWR		: std_logic;
	signal P6_nRD		: std_logic;
	signal P6_nRST		: std_logic;
	signal P6_nRAS2		: std_logic;
	signal P6_nEXCAS	: std_logic;
	signal P6_nDRD2		: std_logic;
	signal P6_4MCLK		: std_logic;
	signal P6_nMPX		: std_logic;
	signal P6_nOCS5		: std_logic;
	signal P6_nWE		: std_logic;
	signal P6_nCS3		: std_logic;
	signal P6_nCS2		: std_logic;
	signal P6_nEXINT	: std_logic;
	signal P6_nEROM		: std_logic;
	signal P6_MONcnt	: std_logic;

	signal doutcnt		: std_logic;
	signal doutcnt_f1	: std_logic;

	signal nocs5_f1		: std_logic;
	signal nocs5_f1m	: std_logic;
	signal nocs5_f2		: std_logic;
	signal ncs2_f1		: std_logic;
	signal ncs2_f1m		: std_logic;
	signal ncs2_f2		: std_logic;
	signal ncs3_f1		: std_logic;
	signal ncs3_f1m		: std_logic;
	signal ncs3_f2		: std_logic;

	signal nras2_f1		: std_logic;
	signal nras2_f1m	: std_logic;
	signal nras2_f2		: std_logic;
	signal nexcas_f1	: std_logic;
	signal nexcas_f1m	: std_logic;
	signal nexcas_f2	: std_logic;
	signal ndrd2_f1		: std_logic;
	signal ndrd2_f1m	: std_logic;
	signal ndrd2_f2		: std_logic;
	signal nwe_f1		: std_logic;
	signal nwe_f1m		: std_logic;
	signal nwe_f2		: std_logic;
	signal mclk_f1		: std_logic;
	signal mclk_f1m		: std_logic;
	signal mclk_f2		: std_logic;

	signal add_f1		: std_logic_vector(15 downto 0);
	signal add_f2		: std_logic_vector(15 downto 0);
	signal d_i_f1		: std_logic_vector(7 downto 0);
	signal d_i_f2		: std_logic_vector(7 downto 0);
	signal nm1_f1		: std_logic;
	signal nm1_f1m		: std_logic;
	signal nm1_f2		: std_logic;
	signal niorq_f1		: std_logic;
	signal niorq_f1m	: std_logic;
	signal niorq_f2		: std_logic;
	signal nmreq_f1		: std_logic;
	signal nmreq_f1m	: std_logic;
	signal nmreq_f2		: std_logic;
	signal nwr_f1		: std_logic;
	signal nwr_f1m		: std_logic;
	signal nwr_f2		: std_logic;
	signal nrd_f1		: std_logic;
	signal nrd_f1m		: std_logic;
	signal nrd_f2		: std_logic;

-- clock generate
	signal rstn			: std_logic;
	signal clk66m		: std_logic;
	signal clk100m		: std_logic;
	signal clk100m_di	: std_logic;
	signal clk1s		: std_logic;
	signal lock_pll		: std_logic;
	signal rstn_initn	: std_logic;
	signal p6_nrst_f0	: std_logic;
	signal p6_nrst_f1	: std_logic;
	signal p6_nrst_f2	: std_logic;
	signal rstcartn_f0	: std_logic;
	signal rstcartn_f1	: std_logic;
	signal rstcartn_f2	: std_logic;
	signal rstcartn_f3	: std_logic;

-- for MEMSEL
	signal memsel_csn		: std_logic;
	signal memsel_csn_f1	: std_logic;
	signal memsel_do		: std_logic_vector(7 downto 0);
	signal sdramadd			: std_logic_vector(19 downto 0);
	signal sdram_rdn		: std_logic;
	signal sdram_wrn		: std_logic;
	signal sram_do			: std_logic_vector(7 downto 0);
	signal sram_enable		: std_logic;
	signal sram_outenb		: std_logic;
	signal pc60mode			: std_logic;
	signal romxmode			: std_logic;
	signal cs01r			: std_logic;
	signal cs01rx			: std_logic;
	signal cscg				: std_logic;

-- for DRAM
	signal dram_di			: std_logic_vector(15 downto 0);
	signal dram_do			: std_logic_vector(15 downto 0);
	signal dram_doenb		: std_logic;

	signal sdram_di			: std_logic_vector(7 downto 0);
	signal sdram_do			: std_logic_vector(7 downto 0);
	signal sdram_memnoinit	: std_logic;
	signal sdram_errrst		: std_logic;
	signal sdram_ready		: std_logic;
	signal sdram_initdone	: std_logic;
	signal sdram_initerr	: std_logic;
	signal sdram_memerr		: std_logic;

-- for SD
	signal SD_DO		: std_logic;
	signal SD_CLK		: std_logic;
	signal SD_DI		: std_logic;
	signal SD_CS		: std_logic;
	signal SD_EJECT		: std_logic;
	signal SD_WP		: std_logic;

-- for OLED
	signal OLED_D0		: std_logic;
	signal OLED_D1		: std_logic;
	signal OLED_CS		: std_logic;
	signal OLED_RST		: std_logic;
	signal OLED_DC		: std_logic;
	signal OLED_SDA		: std_logic;
	signal OLED_SCL		: std_logic;
	signal oled_sdai	: std_logic;

-- for SD I/F
	signal sdio_sd_do			: std_logic;	-- SD
	signal sdio_sd_di			: std_logic;	-- SD
	signal sdio_sd_cs			: std_logic;	-- SD
	signal sdio_sd_clk			: std_logic;	-- SD
	signal sdio_sd_outenb		: std_logic;
	signal sdio_oled_cs			: std_logic;	-- OLED
	signal sdio_oled_rst		: std_logic;	-- OLED
	signal sdio_oled_dc			: std_logic;	-- OLED
	signal sdio_oled_sda		: std_logic;	-- OLED
	signal sdio_oled_scl		: std_logic;	-- OLED
	signal sdio_oled_sdai		: std_logic;	-- OLED
	signal sdio_eeprom_sda		: std_logic;	-- EEPROM
	signal sdio_eeprom_scl		: std_logic;	-- EEPROM
	signal sdio_eeprom_sdai		: std_logic;	-- EEPROM
	signal sdio_oled_i2c		: std_logic;
	signal sdio_csn				: std_logic;
	signal sdio_wrn				: std_logic;
	signal sdio_wrn_f1			: std_logic;
	signal sdio_wrn_f2			: std_logic;
	signal sdio_datao			: std_logic_vector(7 downto 0);
	signal ctrl_userom			: std_logic_vector(1 downto 0);
	signal ctrl_initdone		: std_logic;
	signal ctrl_rstcartn		: std_logic;
	signal ctrl_ledg			: std_logic;
	signal ctrl_ledgblk			: std_logic;
	signal ctrl_ledr			: std_logic;
	signal ctrl_ledrblk			: std_logic;
	signal ctrl_sd_add			: std_logic_vector(19 downto 0);
	signal ctrl_sd_dt			: std_logic_vector(7 downto 0);
	signal ctrl_sd_wrn			: std_logic;
	signal ctrl_bank_add		: std_logic_vector(2 downto 0);
	signal ctrl_bank_dt			: std_logic_vector(7 downto 0);
	signal ctrl_bank_wrn		: std_logic;
	signal rstp6n				: std_logic;
	signal sdio_verno			: std_logic_vector(15 downto 0);

-- control CPU
	signal ctrl_intn	: std_logic;
	signal ctrl_m1n		: std_logic;
	signal ctrl_mreqn	: std_logic;
	signal ctrl_iorqn	: std_logic;
	signal ctrl_rdn		: std_logic;
	signal ctrl_wrn		: std_logic;
	signal ctrl_rfshn	: std_logic;
	signal ctrl_a		: std_logic_vector(15 downto 0);
	signal ctrl_di		: std_logic_vector(7 downto 0);
	signal ctrl_do		: std_logic_vector(7 downto 0);

-- control CPU decode signals
	signal ctrl_memrdn		: std_logic;
	signal ctrl_memwrn		: std_logic;
	signal ctrl_intrdn		: std_logic;

-- ROM/RAM for control CPU
	signal ctrlrom_csn		: std_logic;
	signal ctrlrom_rdn		: std_logic;
	signal ctrlrom_wrn		: std_logic;
	signal ctrlrom_q		: std_logic_vector(7 downto 0);

-- interrupt for control CPU
	signal intgen_do		: std_logic_vector(7 downto 0);

-- PIO
	signal p6pio_csn	: std_logic;
	signal p6pio_csn_f1	: std_logic;
	signal p6pio_pao	: std_logic_vector(7 downto 0);
	signal p6pio_pbo	: std_logic_vector(7 downto 0);
	signal p6pio_pco	: std_logic_vector(3 downto 0);
	signal p6pio_do		: std_logic_vector(7 downto 0);
	signal fdpio_csn	: std_logic;
	signal fdpio_pao	: std_logic_vector(7 downto 0);
	signal fdpio_pbo	: std_logic_vector(7 downto 0);
	signal fdpio_pco	: std_logic_vector(3 downto 0);
	signal fdpio_do		: std_logic_vector(7 downto 0);

-- for LED
	signal ledg			: std_logic;
	signal ledr			: std_logic;
	signal ledb			: std_logic;

-- for Probe
	signal probedat		: std_logic_vector(31 downto 0);
	signal probe2dat	: std_logic_vector(12 downto 0);

	component CLKGEN is
		port (
			CLK50MIN	: in  std_logic;	-- clock 50MHz input #1
			RSTN		: in  std_logic;
			CLK66MOUT	: out std_logic;
			CLK100MOUT	: out std_logic;
			CLK100M_DI	: out std_logic;
			CLK1SOUT	: out std_logic;
			LOCK_PLL	: out std_logic
		);
	end component;

	component SDRAMCNT is
		port (
			DRAM_DI		: in  std_logic_vector(15 downto 0);	-- SDRAM data bus
			ADDRESS		: in  std_logic_vector(19 downto 0);
			INDATA		: in  std_logic_vector(7 downto 0);
			RDN			: in  std_logic;
			WRN			: in  std_logic;
			INIT		: in  std_logic;
			MEMNOINIT	: in  std_logic;
			ERRRST		: in  std_logic;
			CLK100M		: in  std_logic;
			CLK100M_DI	: in  std_logic;
			RSTN		: in  std_logic;
			DRAM_DO		: out std_logic_vector(15 downto 0);	-- SDRAM data bus
			DRAM_DOENB	: out std_logic;						-- SDRAM data output enable
			DRAM_A		: out std_logic_vector(12 downto 0);	-- SDRAM address bus (A12:N.C.)
			DRAM_CLK	: out std_logic;						-- SDRAM clock output
			DRAM_CKE	: out std_logic;						-- SDRAM clock enable
			DRAM_LDQM	: out std_logic;						-- SDRAM LowerByte Data Mask
			DRAM_UDQM	: out std_logic;						-- SDRAM UpperByte Data Mask
			DRAM_WE_N	: out std_logic;						-- SDRAM write Enable
			DRAM_CAS_N	: out std_logic;						-- SDRAM CAS
			DRAM_RAS_N	: out std_logic;						-- SDRAM RAS
			DRAM_CS_N	: out std_logic;						-- SDRAM chip select
			DRAM_BA_1	: out std_logic;						-- SDRAM Bank #1
			DRAM_BA_0	: out std_logic;						-- SDRAM Bank #0
			READY		: out std_logic;
			OUTDATA		: out std_logic_vector(7 downto 0);
			INITDONE	: out std_logic;
			INITERR		: out std_logic;
			MEMERR		: out std_logic
		);
	end component;

	component MEMSEL is
		port (
			CPUADD		: in  std_logic_vector(15 downto 0);
			CPUDT		: in  std_logic_vector(7 downto 0);
			CPUMREQN	: in  std_logic;
			CPUIORQN	: in  std_logic;
			CPUWRN		: in  std_logic;
			CPURDN		: in  std_logic;
			CPUEXCASN	: in  std_logic;
			CPURAS2N	: in  std_logic;
			CPUDRD2N	: in  std_logic;
			CPUWEN		: in  std_logic;
			CSN			: in  std_logic;
			SRAM_ENB	: in  std_logic;
			CTRLRAMADD	: in  std_logic_vector(19 downto 0);
			CTRLRAMWRN	: in  std_logic;
			CTRLBANKADD	: in  std_logic_vector(2 downto 0);
			CTRLBANKDT	: in  std_logic_vector(7 downto 0);
			CTRLBANKWRN	: in  std_logic;
			USEROM		: in  std_logic_vector(1 downto 0);
			RSTP6N		: in  std_logic;
			CLK			: in  std_logic;
			RSTN		: in  std_logic;
			SDRAMADD	: out std_logic_vector(19 downto 0);
			SDRAM_WRN	: out std_logic;
			SDRAM_RDN	: out std_logic;
			PC60MODE	: out std_logic;
			ROMXMODE	: out std_logic;
			CS01R		: out std_logic;
			CS01RX		: out std_logic;
			CSCG		: out std_logic;
			SRAM_OUTENB	: out std_logic;
			SRAM_DO		: out std_logic_vector(7 downto 0);
			DO			: out std_logic_vector(7 downto 0)
		);
	end component;

	component T80s
		generic(
			Mode : integer := 0;	-- 0 => Z80, 1 => Fast Z80, 2 => 8080, 3 => GB
			T2Write : integer := 0;	-- 0 => WR_n active in T3, /=0 => WR_n active in T2
			IOWait : integer := 1	-- 0 => Single cycle I/O, 1 => Std I/O cycle
		);
		port(
			RESET_n		: in std_logic;
			CLK_n		: in std_logic;
			WAIT_n		: in std_logic;
			INT_n		: in std_logic;
			NMI_n		: in std_logic;
			BUSRQ_n		: in std_logic;
			M1_n		: out std_logic;
			MREQ_n		: out std_logic;
			IORQ_n		: out std_logic;
			RD_n		: out std_logic;
			WR_n		: out std_logic;
			RFSH_n		: out std_logic;
			HALT_n		: out std_logic;
			BUSAK_n		: out std_logic;
			A			: out std_logic_vector(15 downto 0);
			DI			: in std_logic_vector(7 downto 0);
			DO			: out std_logic_vector(7 downto 0)
		);
	end component;

	component CTRLROM_WRAP is
		port (
			ADDRESS	: in  std_logic_vector(15 downto 0);
			DATA	: in  std_logic_vector(7 downto 0);
			RDN		: in  std_logic;
			WRN		: in  std_logic;
			CLK		: in  std_logic;
			RSTN	: in  std_logic;
			Q		: out std_logic_vector(7 downto 0)
		);
	end component;

	component INTGEN is
		port (
			RDN		: in  std_logic;
			CLK		: in  std_logic;
			RSTN	: in  std_logic;
			INTRQN	: out std_logic;
			DO		: out std_logic_vector(7 downto 0)
		);
	end component;

	component PIO8255M is
		port (
			A1			: in  std_logic;
			A0			: in  std_logic;
			DI			: in  std_logic_vector(7 downto 0);
			CSN			: in  std_logic;
			RDN			: in  std_logic;
			WRN			: in  std_logic;
			PAI			: in  std_logic_vector(7 downto 0);
			PBI			: in  std_logic_vector(7 downto 0);
			PCI			: in  std_logic_vector(3 downto 0);
			CLK			: in  std_logic;
			RSTN		: in  std_logic;
			DO			: out std_logic_vector(7 downto 0);
			PAO			: out std_logic_vector(7 downto 0);
			PBO			: out std_logic_vector(7 downto 0);
			PCO			: out std_logic_vector(3 downto 0)
		);
	end component;

	component PIO8255MFD is
		port (
			A1			: in  std_logic;
			A0			: in  std_logic;
			DI			: in  std_logic_vector(7 downto 0);
			CSN			: in  std_logic;
			RDN			: in  std_logic;
			WRN			: in  std_logic;
			PAI			: in  std_logic_vector(7 downto 0);
			PBI			: in  std_logic_vector(7 downto 0);
			PCI			: in  std_logic_vector(3 downto 0);
			CLK			: in  std_logic;
			RSTN		: in  std_logic;
			DO			: out std_logic_vector(7 downto 0);
			PAO			: out std_logic_vector(7 downto 0);
			PBO			: out std_logic_vector(7 downto 0);
			PCO			: out std_logic_vector(3 downto 0)
		);
	end component;

	component SDIO is
		port (
			SD_DO			: in  std_logic;	-- from SD
			SD_EJECT		: in  std_logic;	-- from SD
			SD_WP			: in  std_logic;
			RSTP6N			: in  std_logic;
			OLED_SDAI		: in  std_logic;	-- from SD
			EEPROM_SDAI		: in  std_logic;	-- from EEPROM
			OLED_I2C		: in  std_logic;
			VERNO			: in  std_logic_vector(15 downto 0);
			ADD				: in  std_logic_vector(11 downto 0);
			DATAI			: in  std_logic_vector(7 downto 0);
			WRN				: in  std_logic;
			CLK				: in  std_logic;
			RSTN			: in  std_logic;
			SD_DI			: out std_logic;	-- to SD
			SD_CS			: out std_logic;	-- to SD
			SD_CLK			: out std_logic;	-- to SD
			SD_OUTENB		: out std_logic;
			OLED_CS			: out std_logic;	-- to OLED
			OLED_RST		: out std_logic;	-- to OLED
			OLED_DC			: out std_logic;	-- to OLED
			OLED_SCL		: out std_logic;	-- to OLED
			OLED_SDA		: out std_logic;	-- to OLED
			EEPROM_SCL		: out std_logic;	-- to EEPROM
			EEPROM_SDA		: out std_logic;	-- to EEPROM
			DATAO			: out std_logic_vector(7 downto 0);
			USEROM			: out std_logic_vector(1 downto 0);
			INITDONE		: out std_logic;
			RSTCARTN		: out std_logic;
			LEDG			: out std_logic;
			LEDGBLK			: out std_logic;
			LEDR			: out std_logic;
			LEDRBLK			: out std_logic;
			RAMADD			: out std_logic_vector(19 downto 0);	-- to SDRAM
			RAMDT			: out std_logic_vector(7 downto 0);		-- to SDRAM
			RAMWRN			: out std_logic;						-- to SDRAM
			BANKADD			: out std_logic_vector(2 downto 0);
			BANKDT			: out std_logic_vector(7 downto 0);
			BANKWRN			: out std_logic
		);
	end component;

begin

-- HW version number

	sdio_verno <= X"31_33";		-- ver1.3


-- RESET and CLOCK generate

	rstn   <= '0' when (BUTTON(0) = '0') else '1';

	U_CLKGEN : CLKGEN
	port map (
		CLK50MIN	=> CLK50M,
		RSTN		=> rstn,
		CLK66MOUT	=> clk66m,
		CLK100MOUT	=> clk100m,
		CLK100M_DI	=> clk100m_di,
		CLK1SOUT	=> clk1s,
		LOCK_PLL	=> lock_pll
	);

	process (clk100m,rstn)
	begin
		if (rstn = '0') then
			p6_nrst_f0 <= '1';
			p6_nrst_f1 <= '1';
			p6_nrst_f2 <= '1';
		elsif (clk100m'event and clk100m = '1') then
			p6_nrst_f0 <= P6_nRST;
			p6_nrst_f1 <= p6_nrst_f0;
			p6_nrst_f2 <= p6_nrst_f1;
		end if;
	end process;

	rstp6n <= '0' when (ctrl_initdone = '1' and p6_nrst_f2 = '0') else '1';

	process (clk100m,rstn)
	begin
		if (rstn = '0') then
			rstcartn_f0 <= '1';
			rstcartn_f1 <= '1';
			rstcartn_f2 <= '1';
			rstcartn_f3 <= '1';
		elsif (clk100m'event and clk100m = '1') then
			rstcartn_f0 <= ctrl_rstcartn;
			rstcartn_f1 <= rstcartn_f0;
			rstcartn_f2 <= rstcartn_f1;
			rstcartn_f3 <= rstcartn_f2;
		end if;
	end process;

	rstn_initn <= '0' when (lock_pll = '0' or rstcartn_f3 = '0') else '1';

-- data latch from PC-6001
	process (clk100m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			nocs5_f1  <= '1';
			nocs5_f2  <= '1';
			ncs2_f1   <= '1';
			ncs2_f2   <= '1';
			ncs3_f1   <= '1';
			ncs3_f2   <= '1';
			nras2_f1  <= '1';
			nras2_f2  <= '1';
			nexcas_f1 <= '1';
			nexcas_f2 <= '1';
			ndrd2_f1  <= '1';
			ndrd2_f2  <= '1';
			nwe_f1    <= '1';
			nwe_f2    <= '1';
			mclk_f1   <= '1';
			mclk_f2   <= '1';
		elsif (clk100m'event and clk100m = '1') then
			nocs5_f1  <= P6_nOCS5;
			nocs5_f2  <= nocs5_f1m;
			ncs2_f1   <= P6_nCS2;
			ncs2_f2   <= ncs2_f1m;
			ncs3_f1   <= P6_nCS3;
			ncs3_f2   <= ncs3_f1m;
			nras2_f1  <= P6_nRAS2;
			nras2_f2  <= nras2_f1m;
			nexcas_f1 <= P6_nEXCAS;
			nexcas_f2 <= nexcas_f1m;
			ndrd2_f1  <= P6_nDRD2;
			ndrd2_f2  <= ndrd2_f1m;
			nwe_f1    <= P6_nWE;
			nwe_f2    <= nwe_f1m;
			mclk_f1   <= P6_4MCLK;
			mclk_f2   <= mclk_f1m;
		end if;
	end process;

	nocs5_f1m  <= '0' when ( nocs5_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	ncs2_f1m   <= '0' when (  ncs2_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	ncs3_f1m   <= '0' when (  ncs3_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nras2_f1m  <= '0' when ( nras2_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nexcas_f1m <= '0' when (nexcas_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	ndrd2_f1m  <= '0' when ( ndrd2_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nwe_f1m    <= '0' when (   nwe_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	mclk_f1m   <= '0' when (  mclk_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';

	process (clk100m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			add_f1 <= (others => '0');
			add_f2 <= (others => '0');
			d_i_f1 <= (others => '0');
			d_i_f2 <= (others => '0');
		elsif (clk100m'event and clk100m = '1') then
			add_f1 <= P6_A;
			add_f2 <= add_f1;
			d_i_f1 <= P6_D_i;
			d_i_f2 <= d_i_f1;
		end if;
	end process;

	process (clk100m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			nm1_f1   <= '1';
			nm1_f2   <= '1';
			niorq_f1 <= '1';
			niorq_f2 <= '1';
			nmreq_f1 <= '1';
			nmreq_f2 <= '1';
			nwr_f1   <= '1';
			nwr_f2   <= '1';
			nrd_f1   <= '1';
			nrd_f2   <= '1';
		elsif (clk100m'event and clk100m = '1') then
			nm1_f1   <= P6_nM1;
			nm1_f2   <= nm1_f1m;
			niorq_f1 <= P6_nIORQ;
			niorq_f2 <= niorq_f1m;
			nmreq_f1 <= P6_nMREQ;
			nmreq_f2 <= nmreq_f1m;
			nwr_f1   <= P6_nWR;
			nwr_f2   <= nwr_f1m;
			nrd_f1   <= P6_nRD;
			nrd_f2   <= nrd_f1m;
		end if;
	end process;

	nm1_f1m   <= '0' when (  nm1_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	niorq_f1m <= '0' when (niorq_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nmreq_f1m <= '0' when (nmreq_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nwr_f1m   <= '0' when (  nwr_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';
	nrd_f1m   <= '0' when (  nrd_f1 = '0' and ctrl_initdone = '1' and p6_nrst_f2 = '1') else '1';


	U_MEMSEL : MEMSEL
	port map (
		CPUADD		=> add_f2,
		CPUDT		=> d_i_f2,
		CPUMREQN	=> nmreq_f2,
		CPUIORQN	=> niorq_f2,
		CPUWRN		=> nwr_f2,
		CPURDN		=> nrd_f2,
		CPUEXCASN	=> nexcas_f2,
		CPURAS2N	=> nras2_f2,
		CPUDRD2N	=> ndrd2_f2,
		CPUWEN		=> nwe_f2,
		CSN			=> memsel_csn,
		SRAM_ENB	=> sram_enable,
		CTRLRAMADD	=> ctrl_sd_add,
		CTRLRAMWRN	=> ctrl_sd_wrn,
		CTRLBANKADD	=> ctrl_bank_add,
		CTRLBANKDT	=> ctrl_bank_dt,
		CTRLBANKWRN	=> ctrl_bank_wrn,
		USEROM		=> ctrl_userom,
		RSTP6N		=> rstp6n,
		CLK			=> clk100m,
		RSTN		=> rstn_initn,
		SDRAMADD	=> sdramadd,
		SDRAM_WRN	=> sdram_wrn,
		SDRAM_RDN	=> sdram_rdn,
		PC60MODE	=> pc60mode,
		ROMXMODE	=> romxmode,
		CS01R		=> cs01r,
		CS01RX		=> cs01rx,
		CSCG		=> cscg,
		SRAM_OUTENB	=> sram_outenb,
		SRAM_DO		=> sram_do,
		DO			=> memsel_do
	);

	memsel_csn <= niorq_f2 when (add_f2(7 downto 4) = "0011") else '1';

	process (clk100m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			memsel_csn_f1   <= '1';
		elsif (clk100m'event and clk100m = '1') then
			memsel_csn_f1   <= memsel_csn;
		end if;
	end process;


	U_SDRAMCNT : SDRAMCNT
	port map (
		DRAM_DI		=> dram_di,
		ADDRESS		=> sdramadd,
		INDATA		=> sdram_di,
		RDN			=> sdram_rdn,
		WRN			=> sdram_wrn,
		INIT		=> '1',
		MEMNOINIT	=> sdram_memnoinit,
		ERRRST		=> sdram_errrst,
		CLK100M		=> clk100m,
		CLK100M_DI	=> clk100m_di,
		RSTN		=> rstn_initn,
		DRAM_DO		=> dram_do,
		DRAM_DOENB	=> dram_doenb,
		DRAM_A		=> DRAM_A,
		DRAM_CLK	=> DRAM_CLK,
		DRAM_CKE	=> DRAM_CKE,
		DRAM_LDQM	=> DRAM_LDQM,
		DRAM_UDQM	=> DRAM_UDQM,
		DRAM_WE_N	=> DRAM_WE_N,
		DRAM_CAS_N	=> DRAM_CAS_N,
		DRAM_RAS_N	=> DRAM_RAS_N,
		DRAM_CS_N	=> DRAM_CS_N,
		DRAM_BA_1	=> DRAM_BA(1),
		DRAM_BA_0	=> DRAM_BA(0),
		READY		=> sdram_ready,
		OUTDATA		=> sdram_do,
		INITDONE	=> sdram_initdone,
		INITERR		=> sdram_initerr,
		MEMERR		=> sdram_memerr
	);

	DRAM_D  <= dram_do when (dram_doenb = '1') else (others => 'Z');
	dram_di <= DRAM_D;

	sdram_memnoinit <= '0';
	sdram_errrst    <= '0';

	sdram_di <= ctrl_sd_dt when (ctrl_initdone = '0') else d_i_f2;

	P6_D_o <=
		memsel_do when (memsel_csn_f1 = '0' and nrd_f2 = '0') else
		p6pio_do when (p6pio_csn_f1 = '0' and nrd_f2 = '0') else
		sdram_do when (nmreq_f2 = '0' and nexcas_f2 = '0' and nras2_f2 = '0' and ndrd2_f2 = '0') else
		sdram_do when (nmreq_f2 = '0' and nrd_f2 = '0' and (ncs2_f2 = '0' or ncs3_f2 = '0') ) else
		sdram_do when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01r = '1' and pc60mode = '1') else
		sdram_do when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01rx = '1' and cscg = '0' and romxmode = '1') else
		sram_do when (sram_outenb = '1') else
		(others => '1');

	P6_DOUTcnt <=
		'1' when (memsel_csn_f1 = '0' and nrd_f2 = '0') else
		'1' when (p6pio_csn_f1 = '0' and nrd_f2 = '0') else
		'1' when (nmreq_f2 = '0' and nexcas_f2 = '0' and nras2_f2 = '0' and ndrd2_f2 = '0') else
		'1' when (nmreq_f2 = '0' and nrd_f2 = '0' and (ncs2_f2 = '0' or ncs3_f2 = '0') ) else
		'1' when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01r = '1' and pc60mode = '1') else
		'1' when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01rx = '1' and cscg = '0' and romxmode = '1') else
		'1' when (sram_outenb = '1') else
		'0';

	P6_nEROM	<=
		'0' when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01r = '1' and pc60mode = '1') else
		'0' when (nmreq_f2 = '0' and nrd_f2 = '0' and
			add_f2(15 downto 14) = "00" and cs01rx = '1' and cscg = '0' and romxmode = '1') else
		'1';

-- control CPU
	U_T80CTRL : T80s
	generic map (
		Mode 	=> 0,	-- 0 => Z80, 1 => Fast Z80, 2 => 8080, 3 => GB
		T2Write => 0,	-- 0 => WR_n active in T3, /=0 => WR_n active in T2
		IOWait  => 1	-- 0 => Single cycle I/O, 1 => Std I/O cycle
	)
	port map (
		RESET_n		=> sdram_initdone,
		CLK_n		=> clk66m,
		WAIT_n		=> '1',
		INT_n		=> ctrl_intn,
		NMI_n		=> '1',
		BUSRQ_n		=> '1',
		M1_n		=> ctrl_m1n,
		MREQ_n		=> ctrl_mreqn,
		IORQ_n		=> ctrl_iorqn,
		RD_n		=> ctrl_rdn,
		WR_n		=> ctrl_wrn,
		RFSH_n		=> ctrl_rfshn,
		HALT_n		=> open,
		BUSAK_n		=> open,
		A			=> ctrl_a,
		DI			=> ctrl_di,
		DO			=> ctrl_do
	);

	ctrl_memrdn <= '0' when (ctrl_mreqn = '0' and ctrl_rdn = '0') else '1';
	ctrl_memwrn <= '0' when (ctrl_mreqn = '0' and ctrl_wrn = '0') else '1';
	ctrl_intrdn <= '0' when (ctrl_m1n = '0' and ctrl_iorqn = '0') else '1';

	ctrl_di <=	intgen_do     when (ctrl_intrdn = '0') else
				fdpio_do      when (fdpio_csn   = '0') else
				sdio_datao    when (sdio_csn    = '0') else
				ctrlrom_q     when (ctrlrom_csn = '0') else
				(others => '1');


-- control ROM
	U_CTRLROM_WRAP : CTRLROM_WRAP
	port map (
		ADDRESS	=> ctrl_a,
		DATA	=> ctrl_do,
		RDN		=> ctrlrom_rdn,
		WRN		=> ctrlrom_wrn,
		CLK		=> clk66m,
		RSTN	=> rstn_initn,
		Q		=> ctrlrom_q
	);

	ctrlrom_csn <= '0' when (ctrl_a(15 downto 14) /= "11") else '1';
	ctrlrom_rdn <= ctrlrom_csn or ctrl_memrdn;
	ctrlrom_wrn <= ctrlrom_csn or ctrl_memwrn;


-- SW1,2
-- OFF(1),OFF(1)F66/66SR
-- OFF(1), ON(0)Fmk2/mk2SR
--  ON(0),OFF(1)FFDDsgp/փ|[ggpimk2/mk2SRpj
--  ON(0), ON(0)F60

	p6pio_csn <=
		niorq_f2 when (SW(2 downto 1) = "11" and nocs5_f2 = '0') else
		niorq_f2 when (SW(2 downto 1) = "01" and add_f2(7 downto 4) = "1101") else
		niorq_f2 when (SW(2 downto 1) = "10" and add_f2(7 downto 4) = "0001") else
		niorq_f2 when (SW(2 downto 1) = "00" and add_f2(7 downto 4) = "1101") else
		'1';

	sram_enable <= '1' when ( SW(2 downto 1) = "00" ) else '0';


	process (clk100m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			p6pio_csn_f1   <= '1';
		elsif (clk100m'event and clk100m = '1') then
			p6pio_csn_f1   <= p6pio_csn;
		end if;
	end process;


-- interrupt generate for control CPU
	U_INTGEN : INTGEN
	port map (
		RDN		=> ctrl_intrdn,
		CLK		=> clk66m,
		RSTN	=> rstn_initn,
		INTRQN	=> ctrl_intn,
		DO		=> intgen_do
	);


	U_PIO8255M : PIO8255M
	port map (
		A1		=> add_f2(1),
		A0		=> add_f2(0),
		DI		=> d_i_f2,
		CSN		=> p6pio_csn,
		RDN		=> nrd_f2,
		WRN		=> nwr_f2,
		PAI		=> fdpio_pbo,
		PBI		=> fdpio_pao,
		PCI		=> fdpio_pco,
		CLK		=> clk100m,
		RSTN	=> rstp6n,
		DO		=> p6pio_do,
		PAO		=> p6pio_pao,
		PBO		=> p6pio_pbo,
		PCO		=> p6pio_pco
	);

	U_PIO8255MFD : PIO8255MFD
	port map (
		A1		=> ctrl_a(1),
		A0		=> ctrl_a(0),
		DI		=> ctrl_do,
		CSN		=> fdpio_csn,
		RDN		=> ctrl_rdn,
		WRN		=> ctrl_wrn,
		PAI		=> p6pio_pbo,
		PBI		=> p6pio_pao,
		PCI		=> p6pio_pco,
		CLK		=> clk66m,
		RSTN	=> rstn_initn,
		DO		=> fdpio_do,
		PAO		=> fdpio_pao,
		PBO		=> fdpio_pbo,
		PCO		=> fdpio_pco
	);

	fdpio_csn <= ctrl_mreqn when (ctrl_a(15 downto 2) = X"F08" & "00") else '1';	-- 0xf080-0xf083


	U_SDIO : SDIO
	port map (
		SD_DO			=> sdio_sd_do,
		SD_EJECT		=> SD_EJECT,
		SD_WP			=> SD_WP,
		RSTP6N			=> rstp6n,
		OLED_SDAI		=> sdio_oled_sdai,
		EEPROM_SDAI		=> sdio_eeprom_sdai,
		OLED_I2C		=> sdio_oled_i2c,
		VERNO			=> sdio_verno,
		ADD				=> ctrl_a(11 downto 0),
		DATAI			=> ctrl_do,
		WRN				=> sdio_wrn,
		CLK				=> clk100m,
		RSTN			=> rstn_initn,
		SD_DI			=> sdio_sd_di,
		SD_CS			=> sdio_sd_cs,
		SD_CLK			=> sdio_sd_clk,
		SD_OUTENB		=> sdio_sd_outenb,
		OLED_CS			=> sdio_oled_cs,
		OLED_RST		=> sdio_oled_rst,
		OLED_DC			=> sdio_oled_dc,
		OLED_SCL		=> sdio_oled_scl,
		OLED_SDA		=> sdio_oled_sda,
		EEPROM_SCL		=> sdio_eeprom_scl,
		EEPROM_SDA		=> sdio_eeprom_sda,
		DATAO			=> sdio_datao,
		USEROM			=> ctrl_userom,
		INITDONE		=> ctrl_initdone,
		RSTCARTN		=> ctrl_rstcartn,
		LEDG			=> ctrl_ledg,
		LEDGBLK			=> ctrl_ledgblk,
		LEDR			=> ctrl_ledr,
		LEDRBLK			=> ctrl_ledrblk,
		RAMADD			=> ctrl_sd_add,
		RAMDT			=> ctrl_sd_dt,
		RAMWRN			=> ctrl_sd_wrn,
		BANKADD			=> ctrl_bank_add,
		BANKDT			=> ctrl_bank_dt,
		BANKWRN			=> ctrl_bank_wrn
	);

	sdio_csn <= '0' when (ctrl_a(15 downto 12) = "1111") else '1';

	process (clk66m,rstn_initn)
	begin
		if (rstn_initn = '0') then
			sdio_wrn_f1 <= '1';
			sdio_wrn_f2 <= '1';
		elsif (clk66m'event and clk66m = '1') then
			sdio_wrn_f1 <= ctrl_mreqn or ctrl_wrn or sdio_csn;
			sdio_wrn_f2 <= sdio_wrn_f1;
		end if;
	end process;

	sdio_wrn <= sdio_wrn_f1 and sdio_wrn_f2;


	sdio_sd_do <= SD_DO;
	SD_DI      <= sdio_sd_di when (sdio_sd_outenb = '1') else 'Z';
	SD_CS      <= sdio_sd_cs when (sdio_sd_outenb = '1') else 'Z';
	SD_CLK     <= sdio_sd_clk;

	OLED_CS   <= sdio_oled_cs;
	OLED_RST  <= sdio_oled_rst;
	OLED_DC   <= sdio_oled_dc;
	OLED_D1   <= sdio_sd_di;
	OLED_D0   <= sdio_sd_clk;

	OLED_SDA  <= sdio_oled_sda when (sdio_oled_sda = '0') else 'Z';
	OLED_SCL  <= sdio_oled_scl;
	sdio_oled_sdai <= oled_sdai;

	I2C_SDAT <= sdio_eeprom_sda when (sdio_eeprom_sda = '0') else 'Z';
	I2C_SCLK <= sdio_eeprom_scl;
	sdio_eeprom_sdai <= I2C_SDAT;

	ledg <=
		clk1s when (sdram_initerr = '1') else
		'1'   when (rstn_initn = '1' and sdram_initdone = '0') else
		'1'   when (ctrl_ledg = '1') else
		'0';

	ledr <=
		clk1s when (sdram_initerr = '1') else
		'1'   when (rstn_initn = '1' and sdram_initdone = '0') else
		'1'   when (ctrl_ledr = '1') else
		'0';

	ledb <=
		clk1s when (sdram_initerr = '1') else
		clk1s when (sdram_memerr  = '1') else
		'1'   when (ctrl_initdone = '1') else
		'0';

	LED(0) <= ledg;
	LED(1) <= ledr;
	LED(2) <= ledb;

	LED(7 downto 3) <= (others => '0');

	GPIO1_D_06 <= ledg;		-- LED GREEN
	GPIO1_D_07 <= ledr;		-- LED RED
	GPIO1_D_08 <= ledb;		-- LED BLUE

	GPIO2_D_00 <= ledg;		-- LED GREEN
	GPIO2_D_01 <= ledr;		-- LED RED
	GPIO2_D_02 <= ledb;		-- LED BLUE


	probedat(23 downto  0) <= (others => '0');
	probedat(31 downto 24) <= (others => '0');

	P6_nEXINT	<= '1';


-- pin assign for PC-6001 and SD
	P6_A(15)	<= GPIO0_D_08;
	P6_A(14)	<= GPIO0_D_09;
	P6_A(13)	<= GPIO0_D_10;
	P6_A(12)	<= GPIO0_D_11;
	P6_A(11)	<= GPIO0_D_12;
	P6_A(10)	<= GPIO0_D_13;
	P6_A(9)		<= GPIO0_D_24;
	P6_A(8)		<= GPIO0_D_15;
	P6_A(7)		<= GPIO0_D_26;
	P6_A(6)		<= GPIO0_D_17;
	P6_A(5)		<= GPIO0_D_18;
	P6_A(4)		<= GPIO0_D_19;
	P6_A(3)		<= GPIO0_D_20;
	P6_A(2)		<= GPIO0_D_21;
	P6_A(1)		<= GPIO0_D_22;
	P6_A(0)		<= GPIO0_D_23;
	P6_D_i(7)	<= GPIO0_D_00;
	P6_D_i(6)	<= GPIO0_D_01;
	P6_D_i(5)	<= GPIO0_D_02;
	P6_D_i(4)	<= GPIO0_D_03;
	P6_D_i(3)	<= GPIO0_D_04;
	P6_D_i(2)	<= GPIO0_D_05;
	P6_D_i(1)	<= GPIO0_D_06;
	P6_D_i(0)	<= GPIO0_D_07;
	GPIO0_D_00	<= P6_D_o(7) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_01	<= P6_D_o(6) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_02	<= P6_D_o(5) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_03	<= P6_D_o(4) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_04	<= P6_D_o(3) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_05	<= P6_D_o(2) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_06	<= P6_D_o(1) when (P6_DOUTcnt = '1') else 'Z';
	GPIO0_D_07	<= P6_D_o(0) when (P6_DOUTcnt = '1') else 'Z';

	P6_nM1		<= GPIO0_D_33;
	P6_nIORQ	<= GPIO0_D_31;
	P6_nMREQ	<= GPIO0_D_29;
	P6_nWR		<= GPIO0_D_27;
	P6_nRD		<= GPIO0_D_25;
	P6_nRST		<= GPIO0_CLKIN_0;
	P6_nRAS2	<= GPIO0_D_30;
	P6_nEXCAS	<= GPIO0_D_28;
	P6_nDRD2	<= GPIO0_CLKIN_1;
	P6_nMPX		<= GPIO1_D_00;		-- when P6_MONcnt = '0'
	P6_4MCLK	<= GPIO1_D_31;		-- when P6_MONcnt = '0'
	P6_nWE		<= GPIO1_D_01;		-- when P6_MONcnt = '0'
	P6_nOCS5	<= GPIO1_D_33;		-- when P6_MONcnt = '0'
	P6_nCS2		<= GPIO1_D_03;		-- when P6_MONcnt = '0'
	P6_nCS3		<= GPIO1_D_02;		-- when P6_MONcnt = '0'

	GPIO0_D_14	<= P6_DOUTcnt;
	GPIO0_D_16	<= P6_MONcnt;

-- for SD
-- SW3 : OFF(1)F{[hmicroSD	ON(0)FgRlN^SDgp
	GPIO0_D_32	<= SD_CLK        when (SW(3) = '1') else '1';
	GPIO1_D_30	<= SD_DI         when (SW(3) = '1') else '1';
	GPIO1_D_32	<= SD_CS         when (SW(3) = '1') else '1';
	GPIO2_D_03	<= SD_CLK        when (SW(3) = '0') else '1';
	GPIO2_D_04	<= SD_DI         when (SW(3) = '0') else '1';
	GPIO2_D_05	<= SD_CS         when (SW(3) = '0') else '1';
	SD_DO		<= GPIO1_CLKIN_1 when (SW(3) = '1') else GPIO2_CLKIN_0;
	SD_EJECT	<= GPIO1_CLKIN_0 when (SW(3) = '1') else GPIO2_CLKIN_1;
	SD_WP		<= '0'           when (SW(3) = '1') else GPIO2_CLKIN_2;

-- for OLED
-- SW4 : OFF(1)FOLED 4wire SPI	ON(0)FOLED I2C
	GPIO2_D_06	<= OLED_CS;
	GPIO2_D_07	<= OLED_DC;
	GPIO2_D_08	<= OLED_RST;
	GPIO2_D_09	<= OLED_D1		when (SW(4) = '1') else OLED_SDA;
	GPIO2_D_10	<= OLED_D0		when (SW(4) = '1') else OLED_SCL;
	oled_sdai   <= GPIO2_D_09;

	sdio_oled_i2c <= not SW(4);

-- for Probe
	P6_MONcnt <= '0';

	GPIO1_D_00	<= probedat(31) when (P6_MONcnt = '1') else 'Z';
	GPIO1_D_01	<= probedat(29) when (P6_MONcnt = '1') else 'Z';
	GPIO1_D_02	<= probedat(26) when (P6_MONcnt = '1') else 'Z';
	GPIO1_D_03	<= probedat(27) when (P6_MONcnt = '1') else 'Z';
	GPIO1_D_04	<= probedat(24) when (P6_MONcnt = '1') else P6_nEXINT;
	GPIO1_D_05	<= probedat(25) when (P6_MONcnt = '1') else P6_nEROM;
--	GPIO1_D_06	<= probedat(22);
--	GPIO1_D_07	<= probedat(23);
--	GPIO1_D_08	<= probedat(20);
	GPIO1_D_09	<= probedat(21);
	GPIO1_D_10	<= probedat(18);
	GPIO1_D_11	<= probedat(19);
	GPIO1_D_12	<= probedat(16);
	GPIO1_D_13	<= probedat(17);
	GPIO1_D_14	<= probedat(14);
	GPIO1_D_15	<= probedat(15);
	GPIO1_D_16	<= probedat(12);
	GPIO1_D_17	<= probedat(13);
	GPIO1_D_18	<= probedat(10);
	GPIO1_D_19	<= probedat(11);
	GPIO1_D_20	<= probedat(8);
	GPIO1_D_21	<= probedat(9);
	GPIO1_D_22	<= probedat(6);
	GPIO1_D_23	<= probedat(7);
	GPIO1_D_24	<= probedat(4);
	GPIO1_D_25	<= probedat(5);
	GPIO1_D_26	<= probedat(2);
	GPIO1_D_27	<= probedat(3);
	GPIO1_D_28	<= probedat(0);
	GPIO1_D_29	<= probedat(1);
	GPIO1_D_31	<= probedat(30) when (P6_MONcnt = '1') else 'Z';
	GPIO1_D_33	<= probedat(28) when (P6_MONcnt = '1') else 'Z';

-- input only signal
	GPIO0_D_00	<= 'Z';
	GPIO0_D_01	<= 'Z';
	GPIO0_D_02	<= 'Z';
	GPIO0_D_03	<= 'Z';
	GPIO0_D_04	<= 'Z';
	GPIO0_D_05	<= 'Z';
	GPIO0_D_06	<= 'Z';
	GPIO0_D_07	<= 'Z';
	GPIO0_D_08	<= 'Z';
	GPIO0_D_09	<= 'Z';
	GPIO0_D_10	<= 'Z';
	GPIO0_D_11	<= 'Z';
	GPIO0_D_12	<= 'Z';
	GPIO0_D_13	<= 'Z';
	GPIO0_D_15	<= 'Z';
	GPIO0_D_17	<= 'Z';
	GPIO0_D_18	<= 'Z';
	GPIO0_D_19	<= 'Z';
	GPIO0_D_20	<= 'Z';
	GPIO0_D_21	<= 'Z';
	GPIO0_D_22	<= 'Z';
	GPIO0_D_23	<= 'Z';
	GPIO0_D_24	<= 'Z';
	GPIO0_D_25	<= 'Z';
	GPIO0_D_27	<= 'Z';
	GPIO0_D_29	<= 'Z';
	GPIO0_D_31	<= 'Z';
	GPIO0_D_33	<= 'Z';

	GPIO2_D_11	<= 'Z';
	GPIO2_D_12	<= 'Z';

-- unused pin
	G_SENSOR_CS_N	<= '0';
	ADC_CS_N		<= '1';
	ADC_SADDR		<= '0';
	ADC_SCLK		<= '0';

end RTL;
