--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity SDIO is
	port (
		SD_DO			: in  std_logic;	-- from SD
		SD_EJECT		: in  std_logic;	-- from SD
		SD_WP			: in  std_logic;	-- from SD
		RSTP6N			: in  std_logic;
		OLED_SDAI		: in  std_logic;	-- from SD
		EEPROM_SDAI		: in  std_logic;	-- from EEPROM
		OLED_I2C		: in  std_logic;
		VERNO			: in  std_logic_vector(15 downto 0);
		ADD				: in  std_logic_vector(11 downto 0);
		DATAI			: in  std_logic_vector(7 downto 0);
		WRN				: in  std_logic;
		CLK				: in  std_logic;
		RSTN			: in  std_logic;
		SD_DI			: out std_logic;	-- to SD
		SD_CS			: out std_logic;	-- to SD
		SD_CLK			: out std_logic;	-- to SD
		SD_OUTENB		: out std_logic;
		OLED_CS			: out std_logic;	-- to OLED
		OLED_RST		: out std_logic;	-- to OLED
		OLED_DC			: out std_logic;	-- to OLED
		OLED_SCL		: out std_logic;	-- to OLED
		OLED_SDA		: out std_logic;	-- to OLED
		EEPROM_SCL		: out std_logic;	-- to EEPROM
		EEPROM_SDA		: out std_logic;	-- to EEPROM
		DATAO			: out std_logic_vector(7 downto 0);
		USEROM			: out std_logic_vector(1 downto 0);
		INITDONE		: out std_logic;
		RSTCARTN		: out std_logic;
		LEDG			: out std_logic;
		LEDGBLK			: out std_logic;
		LEDR			: out std_logic;
		LEDRBLK			: out std_logic;
		RAMADD			: out std_logic_vector(19 downto 0);	-- to SDRAM
		RAMDT			: out std_logic_vector(7 downto 0);		-- to SDRAM
		RAMWRN			: out std_logic;						-- to SDRAM
		BANKADD			: out std_logic_vector(2 downto 0);		-- to SDRAM
		BANKDT			: out std_logic_vector(7 downto 0);		-- to SDRAM
		BANKWRN			: out std_logic							-- to SDRAM
	);
end SDIO;

architecture RTL of SDIO is

	signal sd_di_i		: std_logic;
	signal sd_cs_i		: std_logic;
	signal sd_clk_i		: std_logic;
	signal sd_outenb_i	: std_logic;
	signal oled_cs_i	: std_logic;
	signal oled_rst_i	: std_logic;
	signal oled_dc_i	: std_logic;
	signal datao_i		: std_logic_vector(7 downto 0);
	signal userom_i		: std_logic_vector(1 downto 0);
	signal initdone_i	: std_logic;
	signal rstcartn_i	: std_logic;
	signal ledg_i		: std_logic;
	signal ledgblk_i	: std_logic;
	signal ledr_i		: std_logic;
	signal ledrblk_i	: std_logic;
	signal ramadd_i		: std_logic_vector(19 downto 0);
	signal ramdt_i		: std_logic_vector(7 downto 0);
	signal ramwrn_i		: std_logic;
	signal bankadd_i	: std_logic_vector(2 downto 0);
	signal bankdt_i		: std_logic_vector(7 downto 0);
	signal bankwrn_i	: std_logic;
	signal ramadd_t		: std_logic_vector(19 downto 0);
	signal ramstenb_f00	: std_logic;
	signal ramstenb_f01	: std_logic;
	signal ramstenb_f02	: std_logic;
	signal ramstenb_f03	: std_logic;
	signal ramstenb_f04	: std_logic;
	signal ramstenb_f05	: std_logic;
	signal ramstenb_f06	: std_logic;
	signal ramstenb_f07	: std_logic;
	signal ramstenb_f08	: std_logic;
	signal ramstenb_f09	: std_logic;
	signal ramstenb_f10	: std_logic;
	signal ramstenb_f11	: std_logic;
	signal ramstenb_f12	: std_logic;
	signal ramstenb_f13	: std_logic;
	signal usidr_i		: std_logic_vector(7 downto 0);
	signal usisft_i		: std_logic;
	signal tim100us_i	: std_logic;
	signal timnx10ms_i	: std_logic;
	signal timnx10_i	: std_logic_vector(7 downto 0);
	signal tim5s_i		: std_logic;

	signal usisft_f0	: std_logic;
	signal usisft_f1	: std_logic;
	signal tim100us_f0	: std_logic;
	signal tim100us_f1	: std_logic;
	signal tim100uscnt	: std_logic_vector(13 downto 0);
	signal timnx10ms_f0	: std_logic;
	signal timnx10ms_f1	: std_logic;
	signal timnx10mscnt	: std_logic_vector(23 downto 0);
	signal timnx10cnt	: std_logic_vector(7 downto 0);
	signal tim5s_f0		: std_logic;
	signal tim5s_f1		: std_logic;
	signal tim5scnt		: std_logic_vector(29 downto 0);
	signal sckoutcnt	: std_logic_vector(3 downto 0);
	signal cntsck		: std_logic_vector(4 downto 0);
	signal sd_do_f1		: std_logic;
	signal sd_do_f2		: std_logic;
	signal eject_f1		: std_logic;
	signal eject_f2		: std_logic;
	signal wp_f1		: std_logic;
	signal wp_f2		: std_logic;
	signal wrn_f1		: std_logic;
	signal wrn_f2		: std_logic;
	signal add_f1		: std_logic_vector(11 downto 0);
	signal add_f2		: std_logic_vector(11 downto 0);
	signal data_f1		: std_logic_vector(7 downto 0);
	signal data_f2		: std_logic_vector(7 downto 0);

	signal i2cdata_ep_i	: std_logic_vector(7 downto 0);
	signal i2cctrl_ep_i	: std_logic_vector(2 downto 0);
	signal i2csft_ep_f0	: std_logic;
	signal i2csft_ep_f1	: std_logic;
	signal i2cack_ep_i	: std_logic;
	signal i2csft_ep_i	: std_logic;
	signal i2ccnt_ep	: std_logic_vector(13 downto 0);
	signal i2cnum_ep	: std_logic_vector(3 downto 0);
	signal i2csdt_ep	: std_logic_vector(7 downto 0);
	signal eeprom_sda_i	: std_logic;
	signal eeprom_scl_i	: std_logic;
	signal i2cdata_ol_i	: std_logic_vector(7 downto 0);
	signal i2cctrl_ol_i	: std_logic_vector(2 downto 0);
	signal i2csft_ol_f0	: std_logic;
	signal i2csft_ol_f1	: std_logic;
	signal i2csft_ol_i	: std_logic;
	signal i2ccnt_ol	: std_logic_vector(13 downto 0);
	signal i2cnum_ol	: std_logic_vector(3 downto 0);
	signal i2csdt_ol	: std_logic_vector(7 downto 0);
	signal oled_sda_i	: std_logic;
	signal oled_scl_i	: std_logic;

begin

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			add_f1  <= (others => '0');
			add_f2  <= (others => '0');
			data_f1 <= (others => '0');
			data_f2 <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			add_f1  <= ADD;
			add_f2  <= add_f1;
			data_f1 <= DATAI;
			data_f2 <= data_f1;
		end if;
	end process;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			wrn_f1 <= '1';
			wrn_f2 <= '1';
		elsif (CLK'event and CLK = '1') then
			if (WRN = '0') then
				wrn_f1 <= '0';
			else
				wrn_f1 <= '1';
			end if;

			wrn_f2 <= wrn_f1;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			initdone_i  <= '0';				-- 0xf000
			ledg_i      <= '0';				-- 0xf001
			ledgblk_i   <= '0';				-- 0xf001
			ledr_i      <= '0';				-- 0xf002
			ledrblk_i   <= '0';				-- 0xf002
			ramdt_i     <= (others => '0');	-- 0xf003
			ramadd_i    <= (others => '0');	-- 0xf004-6
			ramwrn_i    <= '1';				-- 0xf007
			bankdt_i    <= (others => '0');	-- 0xf008
			bankadd_i   <= (others => '0');	-- 0xf009
			bankwrn_i   <= '1';				-- 0xf00a
			ramadd_t    <= (others => '0');	-- 0xf00b
			ramstenb_f00<= '0';				-- 0xf00b
			ramstenb_f01<= '0';				-- 0xf00b
			ramstenb_f02<= '0';				-- 0xf00b
			ramstenb_f03<= '0';				-- 0xf00b
			ramstenb_f04<= '0';				-- 0xf00b
			ramstenb_f05<= '0';				-- 0xf00b
			ramstenb_f06<= '0';				-- 0xf00b
			ramstenb_f07<= '0';				-- 0xf00b
			ramstenb_f08<= '0';				-- 0xf00b
			ramstenb_f09<= '0';				-- 0xf00b
			ramstenb_f10<= '0';				-- 0xf00b
			ramstenb_f11<= '0';				-- 0xf00b
			ramstenb_f12<= '0';				-- 0xf00b
			ramstenb_f13<= '0';				-- 0xf00b
			userom_i    <= (others => '0');	-- 0xf00c
			sd_cs_i     <= '1';				-- 0xf012
			sd_outenb_i <= '0';				-- 0xf013
			timnx10_i   <= (others => '0');	-- 0xf018
			rstcartn_i  <= '1';				-- 0xf021
			oled_cs_i   <= '1';				-- 0xf022
			oled_rst_i  <= '1';				-- 0xf023
			oled_dc_i   <= '0';				-- 0xf024
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1') then
				if    (add_f2 = X"000") then
					initdone_i  <= data_f2(0);
				elsif (add_f2 = X"001") then
					ledg_i      <= data_f2(0);
					ledgblk_i   <= data_f2(1);
				elsif (add_f2 = X"002") then
					ledr_i      <= data_f2(0);
					ledrblk_i   <= data_f2(1);
				elsif (add_f2 = X"003") then
					ramdt_i     <= data_f2;
				elsif (add_f2 = X"004") then
					ramadd_i(7 downto 0)  <= data_f2;
					ramadd_t(7 downto 0)  <= data_f2;
				elsif (add_f2 = X"005") then
					ramadd_i(15 downto 8) <= data_f2;
					ramadd_t(15 downto 8) <= data_f2;
				elsif (add_f2 = X"006") then
					ramadd_i(19 downto 16) <= data_f2(3 downto 0);
					ramadd_t(19 downto 16) <= data_f2(3 downto 0);
				elsif (add_f2 = X"007") then
					ramwrn_i    <= data_f2(0);
				elsif (add_f2 = X"008") then
					bankdt_i    <= data_f2;
				elsif (add_f2 = X"009") then
					bankadd_i   <= data_f2(2 downto 0);
				elsif (add_f2 = X"00a") then
					bankwrn_i   <= data_f2(0);
				elsif (add_f2 = X"00b") then
					ramdt_i     <= data_f2;
					ramstenb_f00<= '1';
				elsif (add_f2 = X"00c") then
					userom_i    <= data_f2(1 downto 0);
				elsif (add_f2 = X"012") then
					sd_cs_i     <= data_f2(0);
				elsif (add_f2 = X"013") then
					sd_outenb_i <= data_f2(0);
				elsif (add_f2 = X"018") then
					timnx10_i   <= data_f2;
				elsif (add_f2 = X"021") then
					rstcartn_i  <= data_f2(0);
				elsif (add_f2 = X"022") then
					oled_cs_i   <= data_f2(0);
				elsif (add_f2 = X"023") then
					oled_rst_i  <= data_f2(0);
				elsif (add_f2 = X"024") then
					oled_dc_i   <= data_f2(0);
				end if;
			else
				if    (ramstenb_f00 = '1' and ramstenb_f01 = '0') then
					ramadd_i <= ramadd_t;
					ramwrn_i <= '0';
				elsif (ramstenb_f12 = '1' and ramstenb_f13 = '0') then
					ramwrn_i <= '1';
					ramadd_t <= ramadd_t + 1;
				end if;
				ramstenb_f00 <= '0';
				ramstenb_f01 <= ramstenb_f00;
				ramstenb_f02 <= ramstenb_f01;
				ramstenb_f03 <= ramstenb_f02;
				ramstenb_f04 <= ramstenb_f03;
				ramstenb_f05 <= ramstenb_f04;
				ramstenb_f06 <= ramstenb_f05;
				ramstenb_f07 <= ramstenb_f06;
				ramstenb_f08 <= ramstenb_f07;
				ramstenb_f09 <= ramstenb_f08;
				ramstenb_f10 <= ramstenb_f09;
				ramstenb_f11 <= ramstenb_f10;
				ramstenb_f12 <= ramstenb_f11;
				ramstenb_f13 <= ramstenb_f12;
			end if;
		end if;
	end process;


	datao_i <=
		usidr_i					when (add_f1 = X"010") else
		"0000000" & usisft_i	when (add_f1 = X"011") else
		"0000000" & sd_cs_i		when (add_f1 = X"012") else
		"0000000" & sd_outenb_i	when (add_f1 = X"013") else
		"0000000" & eject_f2	when (add_f1 = X"014") else
		"0000000" & wp_f2		when (add_f1 = X"015") else
		"0000000" & tim100us_i	when (add_f1 = X"016") else
		"0000000" & timnx10ms_i	when (add_f1 = X"017") else
		timnx10_i				when (add_f1 = X"018") else
		"0000000" & tim5s_i		when (add_f1 = X"019") else
		i2csdt_ep				when (add_f1 = X"01A") else
		"000000" & i2cack_ep_i & i2csft_ep_i	when (add_f1 = X"01B") else
		i2csdt_ol				when (add_f1 = X"01C") else
		"0000000" & i2csft_ol_i	when (add_f1 = X"01D") else
		"0000000" & RSTP6N		when (add_f1 = X"020") else
		"0000000" & OLED_I2C	when (add_f1 = X"025") else
		VERNO(15 downto  8)		when (add_f1 = X"07E") else
		VERNO( 7 downto  0)		when (add_f1 = X"07F") else
		(others => '0');

-- SPI
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			usidr_i <= (others => '1');	-- 0xf010
			sd_di_i <= '1';
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"010") then
				usidr_i <= data_f2;
			elsif (cntsck = 5) then
				usidr_i <= usidr_i(6 downto 0) & sd_do_f2;
			end if;
			if (cntsck = 0) then
				sd_di_i <= usidr_i(7);
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			usisft_f0 <= '0';				-- 0xf010
			usisft_f1 <= '0';				-- 0xf010
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"010") then
				usisft_f0 <= '1';
			else
				usisft_f0 <= '0';
			end if;
			usisft_f1 <= usisft_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			usisft_i  <= '0';				-- 0xf011
			sd_clk_i  <= '1';
			cntsck    <= (others => '1');
			sckoutcnt <= (others => '1');
		elsif (CLK'event and CLK = '1') then
			if (usisft_f0 = '1' and usisft_f1 = '0') then
				sckoutcnt <= (others => '0');
				usisft_i  <= '1';
			elsif (sckoutcnt < 8) then
				if (cntsck >= 9) then
					sckoutcnt <= sckoutcnt + 1;
				end if;
			elsif (usisft_i = '1') then
				usisft_i  <= '0';
			end if;

			if (usisft_f0 = '1' and usisft_f1 = '0') then
				cntsck <= (others => '0');
			elsif (cntsck >= 9) then
				if (sckoutcnt < 7) then
					cntsck <= (others => '0');
				end if;
			else
				cntsck <= cntsck + 1;
			end if;

			if (cntsck = 0) then
				sd_clk_i <= '0';
			elsif (cntsck = 5) then
				sd_clk_i <= '1';
			end if;

		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sd_do_f1 <= '1';
			sd_do_f2 <= '1';
			eject_f1 <= '0';
			eject_f2 <= '0';
			wp_f1    <= '0';
			wp_f2    <= '0';
		elsif (CLK'event and CLK = '1') then
			sd_do_f1 <= SD_DO;
			sd_do_f2 <= sd_do_f1;
			eject_f1 <= SD_EJECT;
			eject_f2 <= eject_f1;
			wp_f1    <= SD_WP;
			wp_f2    <= wp_f1;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			tim100us_f0 <= '0';				-- 0xf016
			tim100us_f1 <= '0';				-- 0xf016
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"016") then
				tim100us_f0 <= data_f2(0);
			else
				tim100us_f0 <= '0';
			end if;
			tim100us_f1 <= tim100us_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			tim100uscnt <= (others => '1');
			tim100us_i  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (tim100us_f0 = '1' and tim100us_f1 = '0') then
				tim100uscnt <= (others => '0');
				tim100us_i  <= '1';
			elsif (tim100uscnt >= 9999) then
				tim100us_i  <= '0';
			else
				tim100uscnt <= tim100uscnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			timnx10ms_f0 <= '0';				-- 0xf017
			timnx10ms_f1 <= '0';				-- 0xf017
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"017") then
				timnx10ms_f0 <= data_f2(0);
			else
				timnx10ms_f0 <= '0';
			end if;
			timnx10ms_f1 <= timnx10ms_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			timnx10mscnt <= (others => '1');
			timnx10ms_i  <= '0';
			timnx10cnt   <= (others => '1');
		elsif (CLK'event and CLK = '1') then
			if (timnx10ms_f0 = '1' and timnx10ms_f1 = '0') then
				timnx10mscnt <= (others => '0');
				timnx10ms_i  <= '1';
				timnx10cnt   <= timnx10_i;
			elsif (timnx10mscnt >= 999999) then
				if (timnx10cnt = 1) then
					timnx10ms_i  <= '0';
				else
					timnx10mscnt <= (others => '0');
					timnx10cnt   <= timnx10cnt - 1;
				end if;
			else
				timnx10mscnt <= timnx10mscnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			tim5s_f0 <= '0';				-- 0xf019
			tim5s_f1 <= '0';				-- 0xf019
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"019") then
				tim5s_f0 <= data_f2(0);
			else
				tim5s_f0 <= '0';
			end if;
			tim5s_f1 <= tim5s_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			tim5scnt <= (others => '1');
			tim5s_i  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (tim5s_f0 = '1' and tim5s_f1 = '0') then
				tim5scnt <= (others => '0');
				tim5s_i  <= '1';
			elsif (tim5scnt >= 499999999) then
				tim5s_i  <= '0';
			else
				tim5scnt <= tim5scnt + 1;
			end if;
		end if;
	end process;


-- I2C
--
-- START=============================
--
--     -----+
-- sda      |
--          +--------------
--
--     ------------+
-- scl             |
--                 +-------
--     2500   5000   2500
---
-- DATA(1bit)=======================
--
--     +---------------------------+
-- sda |                           |
--     +---------------------------+
--
--              +----------+
-- scl          |          |
--     ---------+----------+--------
--       2500      5000      2500		75(1500ns)/150(3000ns)/75(1500ns)
--
-- STOP=============================
--
--                        +---------
-- sda                    |
--     -------------------+
--
--              +-------------------
-- scl          |
--     ---------+           
--        2500     5000    2500
--
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2cdata_ep_i <= (others => '0');			-- 0xf01a
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01a") then
				i2cdata_ep_i <= data_f2;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2cctrl_ep_i <= "000";				-- 0xf01b
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01b") then
				i2cctrl_ep_i <= data_f2(3 downto 1);
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2csft_ep_f0 <= '0';				-- 0xf01b
			i2csft_ep_f1 <= '0';				-- 0xf01b
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01b") then
				i2csft_ep_f0 <= data_f2(0);
			else
				i2csft_ep_f0 <= '0';
			end if;
			i2csft_ep_f1 <= i2csft_ep_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2ccnt_ep   <= (others => '1');
			i2cnum_ep   <= "0000";
			i2cack_ep_i <= '0';
			i2csft_ep_i <= '0';
			eeprom_sda_i <= '1';
			eeprom_scl_i <= '1';
			i2csdt_ep   <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (i2csft_ep_f1 = '1' and i2csft_ep_f0 = '0') then
				i2ccnt_ep   <= (others => '0');
				i2cnum_ep   <= "0000";
				i2cack_ep_i <= '1';
				i2csft_ep_i <= '1';
				i2csdt_ep   <= i2cdata_ep_i;
			elsif (i2csft_ep_i = '1') then
				i2ccnt_ep <= i2ccnt_ep + 1;

				if    (i2cctrl_ep_i(2) = '1') then		-- START
					if    (i2ccnt_ep =    0) then
						eeprom_sda_i <= '1';
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 2499) then
						eeprom_sda_i <= '0';
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 7499) then
						eeprom_sda_i <= '0';
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 9999) then
						i2csft_ep_i <= '0';
					end if;
				elsif (i2cctrl_ep_i(1) = '1') then		-- STOP
					if    (i2ccnt_ep =    0) then
						eeprom_sda_i <= '0';
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 2499) then
						eeprom_sda_i <= '0';
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 7499) then
						eeprom_sda_i <= '1';
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 9999) then
						i2csft_ep_i <= '0';
					end if;
				elsif (i2cctrl_ep_i(0) = '1') then		-- READ
					if    (i2ccnt_ep =    0) then
						eeprom_sda_i <= '1';		-- 'Z'
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 2499) then
						eeprom_sda_i <= '1';		-- 'Z' / No Ack
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 4999) then
						if (i2cnum_ep /= 8) then
							i2csdt_ep <= i2csdt_ep(6 downto 0) & EEPROM_SDAI;
						end if;
					elsif (i2ccnt_ep = 7499) then
						eeprom_sda_i <= '1';		-- 'Z'
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 9999) then
						if (i2cnum_ep = 8) then
							i2csft_ep_i <= '0';
						else
							i2ccnt_ep <= (others => '0');
							i2cnum_ep <= i2cnum_ep + 1;
						end if;
					end if;
				else									-- WRITE
					if    (i2ccnt_ep =    0) then
						eeprom_sda_i <= i2csdt_ep(7);
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 2499) then
						eeprom_sda_i <= i2csdt_ep(7);
						eeprom_scl_i <= '1';
					elsif (i2ccnt_ep = 4999) then
						if (i2cnum_ep = 8) then
							i2cack_ep_i <= EEPROM_SDAI;
						end if;
					elsif (i2ccnt_ep = 7499) then
						eeprom_sda_i <= i2csdt_ep(7);
						eeprom_scl_i <= '0';
					elsif (i2ccnt_ep = 9999) then
						if (i2cnum_ep = 8) then
							i2csft_ep_i <= '0';
						else
							i2ccnt_ep <= (others => '0');
							i2cnum_ep <= i2cnum_ep + 1;
							i2csdt_ep <= i2csdt_ep(6 downto 0) & '1';
						end if;
					end if;
				end if;

			end if;
		end if;
	end process;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2cdata_ol_i <= (others => '0');			-- 0xf01c
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01c") then
				i2cdata_ol_i <= data_f2;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2cctrl_ol_i <= "000";				-- 0xf01d
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01d") then
				i2cctrl_ol_i <= data_f2(3 downto 1);
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2csft_ol_f0 <= '0';				-- 0xf01d
			i2csft_ol_f1 <= '0';				-- 0xf01d
		elsif (CLK'event and CLK = '1') then
			if (wrn_f1 = '0' and wrn_f2 = '1' and add_f2 = X"01d") then
				i2csft_ol_f0 <= data_f2(0);
			else
				i2csft_ol_f0 <= '0';
			end if;
			i2csft_ol_f1 <= i2csft_ol_f0;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			i2ccnt_ol   <= (others => '1');
			i2cnum_ol   <= "0000";
			i2csft_ol_i <= '0';
			oled_sda_i <= '1';
			oled_scl_i <= '1';
			i2csdt_ol   <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (i2csft_ol_f1 = '1' and i2csft_ol_f0 = '0') then
				i2ccnt_ol   <= (others => '0');
				i2cnum_ol   <= "0000";
				i2csft_ol_i <= '1';
				i2csdt_ol   <= i2cdata_ol_i;
			elsif (i2csft_ol_i = '1') then
				i2ccnt_ol <= i2ccnt_ol + 1;

				if    (i2cctrl_ol_i(2) = '1') then		-- START
					if    (i2ccnt_ol =   0) then
						oled_sda_i <= '1';
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 149) then
						oled_sda_i <= '0';
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 449) then
						oled_sda_i <= '0';
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 599) then
						i2csft_ol_i <= '0';
					end if;
				elsif (i2cctrl_ol_i(1) = '1') then		-- STOP
					if    (i2ccnt_ol =   0) then
						oled_sda_i <= '0';
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 149) then
						oled_sda_i <= '0';
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 449) then
						oled_sda_i <= '1';
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 599) then
						i2csft_ol_i <= '0';
					end if;
				elsif (i2cctrl_ol_i(0) = '1') then		-- READ
					if    (i2ccnt_ol =   0) then
						oled_sda_i <= '1';		-- 'Z'
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 149) then
						oled_sda_i <= '1';		-- 'Z' / No Ack
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 299) then
						if (i2cnum_ol /= 8) then
							i2csdt_ol <= i2csdt_ol(6 downto 0) & OLED_SDAI;
						end if;
					elsif (i2ccnt_ol = 449) then
						oled_sda_i <= '1';		-- 'Z'
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 599) then
						if (i2cnum_ol = 8) then
							i2csft_ol_i <= '0';
						else
							i2ccnt_ol <= (others => '0');
							i2cnum_ol <= i2cnum_ol + 1;
						end if;
					end if;
				else									-- WRITE
					if    (i2ccnt_ol =   0) then
						oled_sda_i <= i2csdt_ol(7);
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 149) then
						oled_sda_i <= i2csdt_ol(7);
						oled_scl_i <= '1';
					elsif (i2ccnt_ol = 449) then
						oled_sda_i <= i2csdt_ol(7);
						oled_scl_i <= '0';
					elsif (i2ccnt_ol = 599) then
						if (i2cnum_ol = 8) then
							i2csft_ol_i <= '0';
						else
							i2ccnt_ol <= (others => '0');
							i2cnum_ol <= i2cnum_ol + 1;
							i2csdt_ol <= i2csdt_ol(6 downto 0) & '1';
						end if;
					end if;
				end if;

			end if;
		end if;
	end process;


	SD_DI      <= sd_di_i;
	SD_CS      <= sd_cs_i;
	SD_CLK     <= sd_clk_i;
	SD_OUTENB  <= sd_outenb_i;
	OLED_CS    <= oled_cs_i;
	OLED_RST   <= oled_rst_i;
	OLED_DC    <= oled_dc_i;
	OLED_SCL   <= oled_scl_i;
	OLED_SDA   <= oled_sda_i;
	EEPROM_SCL <= eeprom_scl_i;
	EEPROM_SDA <= eeprom_sda_i;
	DATAO      <= datao_i;
	USEROM     <= userom_i;
	INITDONE   <= initdone_i;
	RSTCARTN   <= rstcartn_i;
	LEDG       <= ledg_i;
	LEDGBLK    <= ledgblk_i;
	LEDR       <= ledr_i;
	LEDRBLK    <= ledrblk_i;
	RAMADD     <= ramadd_i;
	RAMDT      <= ramdt_i;
	RAMWRN     <= ramwrn_i;
	BANKADD    <= bankadd_i;
	BANKDT     <= bankdt_i;
	BANKWRN    <= bankwrn_i;

end RTL;
