#include <stdio.h>
#include <string.h>

#include "pc6031.h"
#include "oled.h"
#include "spi.h"
#include "font8x12.h"
#include "verdata.h"

#define SSD1306_DISP_CONTRAST		0x81
#define SSD1306_DISP_ENTIREOFF		0xA4
#define SSD1306_DISP_ENTIREON		0xA5
#define SSD1306_DISP_NOINV			0xA6
#define SSD1306_DISP_INV			0xA7
#define SSD1306_DISP_OFF			0xAE
#define SSD1306_DISP_ON				0xAF
#define SSD1306_DISP_STARTLINE		0x40	/* + 0x00-0x3F */
#define SSD1306_DISP_ENABLELINE		0xA8
#define SSD1306_DISP_COLUMN_NOINV	0xA0
#define SSD1306_DISP_COLUMN_INV		0xA1
#define SSD1306_DISP_ROW_NOINV		0xC0
#define SSD1306_DISP_ROW_INV		0xC8
#define SSD1306_DISP_ROW_OFFSET		0xD3
#define SSD1306_DISP_COMPINS		0xDA

#define SSD1306_ADD_MODESET			0x20	/* 00:H-add 01:V-add 10:PAGE-add */
#define SSD1306_ADD_COLUMN_LIMIT	0x21	/* H/V-add only */
#define SSD1306_ADD_PAGE_LIMIT		0x22	/* H/V-add only */
#define SSD1306_ADD_COLUMN_LOWER	0x00	/* + 0x00-0x0F */ /* PAGE-add only */
#define SSD1306_ADD_COLUMN_UPPER	0x10	/* + 0x00-0x0F */ /* PAGE-add only */
#define SSD1306_ADD_PAGE			0xB0	/* + 0x00-0x07 */ /* PAGE-add only */

#define SSD1306_MISC_CLKDIV			0xD5
#define SSD1306_MISC_PRECHARGE		0xD9
#define SSD1306_MISC_DESELECT_VOL	0xDB
#define SSD1306_MISC_CHARGEPUMP		0x8D

BYTE oled_init_cmd[] = {
	SSD1306_DISP_OFF,
	SSD1306_MISC_CLKDIV,		0x80,
	SSD1306_MISC_PRECHARGE,		0xF1,
	SSD1306_MISC_DESELECT_VOL,	0x40,
	SSD1306_MISC_CHARGEPUMP,	0x14,
	SSD1306_ADD_MODESET,		0x00,			/* horizontal address mode */
	SSD1306_ADD_PAGE_LIMIT,		0x00, 0x07,		/* cJnʒu, Iʒu */
	SSD1306_ADD_COLUMN_LIMIT,	0x00, 0x7f,		/* Jnʒu, Iʒu */
	SSD1306_DISP_COMPINS,		0x12,
	SSD1306_DISP_ENABLELINE,	0x3F,
	SSD1306_DISP_ROW_OFFSET,	0x00,
	SSD1306_DISP_COLUMN_NOINV,
	SSD1306_DISP_ROW_INV,
	SSD1306_DISP_STARTLINE,				/* +0x00 */
	SSD1306_DISP_CONTRAST, 		0xCF, /* contrast value */
	SSD1306_DISP_ENTIREOFF,
	SSD1306_DISP_NOINV,
	SSD1306_DISP_ON	
};

extern DISK_t	diskp;
BYTE	drawmode;

void oled_command(BYTE dt)
{
#if PC6031_OLED_USE_I2C
if ( peek(OLED_I2C) & 0x01 ) {
	poke(I2CDATA_OL, 0x00);
	poke(I2CCTRL_OL, 0b00001001);			/* STARTo */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, OLED_I2C_ADDRESS+0);	/* I2CAhX */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, OLED_I2C_COMMAND);		/* R}h */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, dt);					/* ݃f[^ */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, 0x00);
	poke(I2CCTRL_OL, 0b00000101);			/* STOPo */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}
}
#endif /* PC6031_OLED_USE_I2C */
if ( !(peek(OLED_I2C) & 0x01) ) {
	OLED_CS_HIGH();
	OLED_SET_COMMAND();
	OLED_CS_LOW();
	xmit_spi(dt);
	OLED_CS_HIGH();
	rcv_spi();		/* dummy read */
}
}

void oled_data(BYTE dt)
{
#if PC6031_OLED_USE_I2C
if ( peek(OLED_I2C) & 0x01 ) {
	poke(I2CDATA_OL, 0x00);
	poke(I2CCTRL_OL, 0b00001001);			/* STARTo */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, OLED_I2C_ADDRESS+0);	/* I2CAhX */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, OLED_I2C_DATA);		/* f[^ */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, dt);					/* ݃f[^ */
	poke(I2CCTRL_OL, 0b00000001);			/* o */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}

	poke(I2CDATA_OL, 0x00);
	poke(I2CCTRL_OL, 0b00000101);			/* STOPo */
	while ( peek(I2CCTRL_OL) & 0x01 ) {		/* o܂ő҂ */
		;
	}
}
#endif /* PC6031_OLED_USE_I2C */
if ( !(peek(OLED_I2C) & 0x01) ) {
	OLED_CS_HIGH();
	OLED_SET_DATA();
	OLED_CS_LOW();
	xmit_spi(dt);
	OLED_CS_HIGH();
	rcv_spi();		/* dummy read */
}
}

void oled_init(void)
{
	UINT	i,cmdnum;

#if PC6031_OLED_USE_I2C
if ( peek(OLED_I2C) & 0x01 ) {
	;
}
#endif /* PC6031_OLED_USE_I2C */
if ( !(peek(OLED_I2C) & 0x01) ) {
	OLED_RST_HIGH();
	_delay_ms(10);
	OLED_RST_LOW();
	_delay_ms(10);
	OLED_RST_HIGH();
}

	cmdnum = sizeof(oled_init_cmd) / sizeof(BYTE);
	for (i = 0;i < cmdnum;i ++) {
		oled_command(oled_init_cmd[i]);
	}

	for (i = 0;i < 128*8;i ++) {
		oled_data(0x00);
	}
	drawmode = 0;
}

void oled_setline(BYTE dt)
{
	if ( drawmode ) {
		return;
	}

	oled_command(SSD1306_ADD_PAGE_LIMIT);		/* c  Jnʒu̕ύX */
	oled_command(0);							/* cJnʒu */
	oled_command(7);							/* cIʒu */
	oled_command(SSD1306_ADD_COLUMN_LIMIT);		/*   Jnʒu̕ύX */
	oled_command(FONT_HEIGHT*dt);				/* Jnʒu */
	oled_command(FONT_HEIGHT*(dt+1)-1);			/* Iʒu */
}

void oled_putcdt(BYTE dt,BYTE xdt)
{
	BYTE	i;

	if ( drawmode ) {
		return;
	}

	if (dt < 0x20) {
		dt = 0x7f;
	}
	for (i = 0;i < FONT_HEIGHT;i ++) {
		oled_data( xdt ^ ( font[ FONT_HEIGHT*(dt-0x20)+i ] ) );
	}
}

void oled_putsdt(char *dt, BYTE xdt)
{
	BYTE	i;

	if ( drawmode ) {
		return;
	}

	for (i = 0;i < 8;i ++) {
		if (*dt == 0) {
			break;
		}
		oled_putcdt(*dt++, xdt);
	}
	for (;i < 8;i ++) {
		oled_putcdt(' ', xdt);
	}
}

void oled_dispver(char *dt)
{
	char	tmpname[9];
	UINT	i;

	if ( drawmode ) {
		return;
	}

	sprintf(tmpname, "%c%cv", HW_VERNO );
	strncat(tmpname, dt, 6);
	oled_setline(0);
	oled_puts(tmpname);

	oled_command(SSD1306_ADD_PAGE_LIMIT);		/* c  Jnʒu̕ύX */
	oled_command(0);							/* cJnʒuicJnʒuj*/
	oled_command(7);							/* cIʒu */
	oled_command(SSD1306_ADD_COLUMN_LIMIT);		/*   Jnʒu̕ύX */
	oled_command(FONT_HEIGHT*1);				/* JnʒuiJnʒuj*/
	oled_command(127);							/* Iʒu */
	for (i = FONT_HEIGHT*1*4;i < 128*8;i ++) {
		oled_data(0x00);
	}

	oled_command(SSD1306_ADD_PAGE_LIMIT);		/* c  Jnʒu̕ύX */
	oled_command(0);							/* cJnʒuicJnʒuj*/
	oled_command(7);							/* cIʒu */
	oled_command(SSD1306_ADD_COLUMN_LIMIT);		/*   Jnʒu̕ύX */
	oled_command(FONT_HEIGHT*1);				/* JnʒuiJnʒuj*/
	oled_command(FONT_HEIGHT*5-1);				/* Iʒu */
	for (i = 0;i < FONT_HEIGHT*4*8;i ++) {
		oled_data( verpict[i] );
	}

}

void oled_dispfile(void)
{
	UINT	i;
	BYTE	xdt;
	char	tmpname[9];

	if ( drawmode ) {
		return;
	}

	for (i = 0;i < DRVNUM;i ++) {

		if (i != ROMDRIVE) {
			oled_setline(i*2+0);
			if ( !diskp.fname[i][0] ) {
				tmpname[0] = 0x85;	/*  */
				tmpname[1] = 0xcf;	/* } */
				tmpname[2] = 0xb3;	/* E */
				tmpname[3] = 0xdd;	/*  */
				tmpname[4] = 0xc4;	/* g */
				tmpname[5] = 0xe5;	/*  */
				tmpname[6] = 0x9c;	/*  */
				tmpname[7] = 0x85;	/*  */
				tmpname[8] = 0;
				oled_puts(tmpname);
			} else {
				xdt = ( diskp.writep[i] ) ? 0xff : 0x00;
				oled_putsdt( get_fname( tmpname, diskp.fname[i] ), xdt );
			}
		} else {
#if PC6031_ROM_USE
			oled_setline(9);
			if ( !diskp.rombody[0] ) {
				tmpname[0] = 0x85;	/*  */
				tmpname[1] = 0xcf;	/* } */
				tmpname[2] = 0xb3;	/* E */
				tmpname[3] = 0xdd;	/*  */
				tmpname[4] = 0xc4;	/* g */
				tmpname[5] = 0xe5;	/*  */
				tmpname[6] = 0x9c;	/*  */
				tmpname[7] = 0x85;	/*  */
				tmpname[8] = 0;
				oled_puts(tmpname);
			} else {
				oled_putsdt( diskp.rombody, 0x00 );
			}
#endif /* PC6031_ROM_USE */
		}


		if (i != ROMDRIVE) {
			oled_setline(i*2+1);
			strcpy( tmpname, ".");
			strncat(tmpname, diskp.fext[i], 3);
			strncat(tmpname, " ", 1);
			strncat(tmpname, (( diskp.mode[i] ) ? "1DD" : "1D "), 3);
			oled_puts(tmpname);
		}
	}

	oled_setline(4);
	oled_puts("        ");
}

void oled_drawmode(void)
{
	BYTE	i,j;

	if ( drawmode ) {
		return;
	}
	drawmode = 1;

	oled_command(SSD1306_ADD_PAGE_LIMIT);		/* c  Jnʒu̕ύX */
	oled_command(0);							/* cJnʒuicJnʒuj*/
	oled_command(7);							/* cIʒu */
	oled_command(SSD1306_ADD_COLUMN_LIMIT);		/*   Jnʒu̕ύX */
	oled_command(0);							/* JnʒuiJnʒuj*/
	oled_command(127);							/* Iʒu */

	oled_command(SSD1306_ADD_MODESET);
	oled_command(2);							/* PAGE address */

	for (j = 0;j < 8;j ++) {
		oled_command(SSD1306_ADD_COLUMN_LOWER);	/* column address */
		oled_command(SSD1306_ADD_COLUMN_UPPER);	/* column address */
		oled_command(SSD1306_ADD_PAGE + j);		/* row address */
		for (i = 0;i < 128;i ++) {
			oled_data(0x00);
		}
	}

}

void oled_fontmode(void)
{
	UINT	i;


	if ( !drawmode ) {
		return;
	}
	drawmode = 0;

	oled_command(SSD1306_DISP_NOINV);			/* ]߂ */
	oled_command(SSD1306_ADD_MODESET);
	oled_command(0);							/* horizontal address */

	oled_command(SSD1306_ADD_PAGE_LIMIT);		/* c  Jnʒu̕ύX */
	oled_command(0);							/* cJnʒuicJnʒuj*/
	oled_command(7);							/* cIʒu */
	oled_command(SSD1306_ADD_COLUMN_LIMIT);		/*   Jnʒu̕ύX */
	oled_command(0);							/* JnʒuiJnʒuj*/
	oled_command(127);							/* Iʒu */

	for (i = 0;i < 128*8;i ++) {
		oled_data(0x00);
	}

	oled_dispfile();
}

void oled_drawinv(void)
{
	if ( !drawmode ) {
		return;
	}
	oled_command(SSD1306_DISP_INV);		/* ] */
}

void oled_drawnoinv(void)
{
	if ( !drawmode ) {
		return;
	}
	oled_command(SSD1306_DISP_NOINV);	/* ]߂ */
}

void oled_setcursor(BYTE row, BYTE clm)
{
	if ( !drawmode ) {
		return;
	}

	if (row < 8) {
		oled_command(SSD1306_ADD_PAGE + row);		/* row address */
	}

	if (clm < 128) {
		oled_command(SSD1306_ADD_COLUMN_LOWER +   (clm & 0x0f) );		/* column address */
		oled_command(SSD1306_ADD_COLUMN_UPPER + ( (clm & 0x70) >> 4) );	/* column address */
	}
}

void oled_setdata(BYTE dt)
{
	if ( !drawmode ) {
		return;
	}
	oled_data(dt);
}

